/***************************************************************************
                          zscreen.cpp  -  description
                             -------------------
    begin                : Wed Aug 21 01:49:10 EDT 2002
    copyright            : (C) 2002 by Kevin Greenhaw
    email                : kevin_greenhaw@email.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qfont.h>

#include "endscreen.h"
#include "finish_image.h"
#include "grid.h"
#include "zrally_images.h"

const int LINE1 = 180;
const int LINE2 = 195;
const int LINE3 = 210;
const int LINE4 = 225;
const int LINE5 = 240;

/*****************************************************************************/
/* Function:     EndScreen                                                   */ 
/* Description:  constructor                                                 */
/* Limitations:  None                                                        */
/* Return Value: None                                                        */
/*****************************************************************************/
EndScreen::EndScreen(QCanvas& c, QWidget* parent, const char* name) 
   : QWidget( parent, name ),
     canvas(c)
{
   QRect  imgRect;

   editor = new QCanvasView(&canvas,this);
   editor->setGeometry(8, -1, 225, 292);
   editor->setVScrollBarMode(QScrollView::AlwaysOff);
   editor->setHScrollBarMode(QScrollView::AlwaysOff);
   editor->setFrameStyle( QFrame::NoFrame );
   
   RestartString = new QCanvasText("Hit Any Key To Continue", &canvas);
   RestartString->setFont(QFont("Helvetica", 10, QFont::Bold, false));
   RestartString->setColor(QColor(0x00, 0x00, 0x00));
   
   imgRect = RestartString->boundingRect();
   RestartString->setX( (canvas.width()/2) - (imgRect.width()/2) );
   RestartString->setY(LINE5);
   RestartString->setZ(4);
   RestartString->hide();
}

/*****************************************************************************/
/* Function:     Initialize                                                  */ 
/* Description:                                                              */
/* Limitations:  None                                                        */
/* Return Value: None                                                        */
/*****************************************************************************/
void EndScreen::Show(bool   Winner, 
                     int    RaceTrack, 
                     int    Laps, 
                     double FinishTime, 
                     double RecordTime)
{
   QImage  img;
   QRect   imgRect;
   QString LapsString;
   QString RaceTimeString;
   QString ALSString;
   QString RecordSpeedString;

   QFont textFont("Helvetica", 10, QFont::Normal, false);
   QColor textColor(0x00, 0x00, 0x00);

   if (Winner)
   {
      img = zrallyImages->GetImage(ZRallyImages::RACE_WON);
   }
   else
   {
      img = zrallyImages->GetImage(ZRallyImages::RACE_LOST);
   }

   FinishImage = new ImageItem(img, &canvas);

   imgRect = FinishImage->boundingRect();

   FinishImage->setX( (canvas.width()/2) - (imgRect.width()/2) );
   FinishImage->setY(-1);
   FinishImage->setZ(3);
   FinishImage->show();

   LapsString.sprintf("Number of Laps: %d", Laps);
   RaceTimeString.sprintf("Race Time: %3.3f", FinishTime);

   if ( FinishTime == RecordTime )
   {
      RecordSpeedString = "A New Record";
   }
   else
   {
      RecordSpeedString.sprintf("Track Record: %3.3f", RecordTime);
   }

   LapsText = new QCanvasText(LapsString, &canvas);
   LapsText->setFont(textFont);
   LapsText->setColor(textColor);

   imgRect = LapsText->boundingRect();
   LapsText->setX( (canvas.width()/2) - (imgRect.width()/2) );
   LapsText->setY(LINE1);
   LapsText->setZ(4);
   LapsText->show();

   FinishTimeText = new QCanvasText(RaceTimeString, &canvas);
   FinishTimeText->setFont(textFont);
   FinishTimeText->setColor(textColor);

   imgRect = FinishTimeText->boundingRect();
   FinishTimeText->setX( (canvas.width()/2) - (imgRect.width()/2) );
   FinishTimeText->setY(LINE2);
   FinishTimeText->setZ(4);
   FinishTimeText->show();

   RecordText = new QCanvasText(RecordSpeedString, &canvas);
   RecordText->setFont(textFont);
   RecordText->setColor(textColor);

   imgRect = RecordText->boundingRect();
   RecordText->setX( (canvas.width()/2) - (imgRect.width()/2) );
   RecordText->setY(LINE3);
   RecordText->setZ(4);
   RecordText->show();

/*   
   ALSStringText = new QCanvasText(ALSString, &canvas);
   ALSStringText->setFont(textFont);
   ALSStringText->setColor(textColor);

   imgRect = ALSStringText->boundingRect();
   ALSStringText->setX( (canvas.width()/2) - (imgRect.width()/2) );
   ALSStringText->setY(LINE4);
   ALSStringText->setZ(4);
   ALSStringText->show();
*/
   RestartString->show();
   canvas.update();
   show();
}

/*****************************************************************************/
/* Function:     Hide                                                        */ 
/* Description:                                                              */
/* Limitations:  None                                                        */
/* Return Value: None                                                        */
/*****************************************************************************/
void EndScreen::Hide()
{
//   delete ALSStringText;
   delete FinishTimeText;
   delete LapsText;
   delete FinishImage;
   delete RecordText;

   RestartString->hide();
   hide();
}

/*****************************************************************************/
/* Function:     keyReleaseEvent                                             */ 
/* Description:                                                              */
/* Limitations:  None                                                        */
/* Return Value: None                                                        */
/*****************************************************************************/
void EndScreen::keyPressEvent()
{
   emit ReturnToStart();
}

