/***************************************************************************
                          grid.cpp  -  description
                             -------------------
    begin                : Wed Aug 21 01:49:10 EDT 2002
    copyright            : (C) 2002 by Kevin Greenhaw
    email                : kevin_greenhaw@email.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include <qpainter.h>
#include "grid.h"

#include "VerticalTrack.h"
#include "VerticalBridge.h"
#include "VerticalStart.h"
#include "VerticalStartBridge.h"
#include "HorizontalTrack.h"
#include "HorizontalBridge.h"
#include "UpperLeft.h"
#include "UpperRight.h"
#include "LowerLeft.h"
#include "LowerRight.h"
#include "Grass.h"

//***************************************************************************//
//************************ ImageItem Class **********************************//
//***************************************************************************//
/*****************************************************************************/
/* Function:     ImageItem                                                   */ 
/* Description:  Constructor                                                 */
/* Limitations:  None                                                        */
/* Return Value: None                                                        */
/*****************************************************************************/
ImageItem::ImageItem( QImage img, QCanvas *canvas )
    : QCanvasRectangle( canvas ), image(img)
{
    setSize( image.width(), image.height() );

#ifndef _WS_QWS_
    pixmap.convertFromImage(image, OrderedAlphaDither);
#endif
}

/*****************************************************************************/
/* Function:     ImageItem                                                   */ 
/* Description:  Constructor                                                 */
/* Limitations:  None                                                        */
/* Return Value: None                                                        */
/*****************************************************************************/
ImageItem::ImageItem( QPixmap img, QCanvas *canvas )
    : QCanvasRectangle( canvas ), pixmap(img)
{
    setSize( pixmap.width(), pixmap.height() );

#ifndef _WS_QWS_
    image = pixmap.convertToImage();
#endif
}

/*****************************************************************************/
/* Function:     ImageItem                                                   */ 
/* Description:  Constructor                                                 */
/* Limitations:  None                                                        */
/* Return Value: None                                                        */
/*****************************************************************************/
ImageItem::ImageItem( QCanvas *canvas )
    : QCanvasRectangle( canvas )
{
}

/*****************************************************************************/
/* Function:     SetImage                                                    */ 
/* Description:                                                              */
/* Limitations:  None                                                        */
/* Return Value: None                                                        */
/*****************************************************************************/
void ImageItem::SetImage()
{
   move(Min_XImageValue, Min_YImageValue);
   show();
}

/*****************************************************************************/
/* Function:     SetImage                                                    */ 
/* Description:                                                              */
/* Limitations:  None                                                        */
/* Return Value: None                                                        */
/*****************************************************************************/
void ImageItem::SetImage(int XCoord, int YCoord)
{
   move(XCoord * pixmap.width(), YCoord * pixmap.height());
   show();
}

/*****************************************************************************/
/* Function:     drawShape                                                   */ 
/* Description:                                                              */
/* Limitations:  None                                                        */
/* Return Value: None                                                        */
/*****************************************************************************/
void ImageItem::drawShape( QPainter &p )
{
// On Qt/Embedded, we can paint a QImage as fast as a QPixmap,
// but on other platforms, we need to use a QPixmap.
#ifdef _WS_QWS_
   p.drawImage( int(x()), int(y()), image, 0, 0, -1, -1, OrderedAlphaDither );
#else
   p.drawPixmap( int(x()), int(y()), pixmap );
#endif
}


//***************************************************************************//
//************************ GridSection Class ********************************//
//***************************************************************************//
/*****************************************************************************/
/* Function:     GridSection                                                 */ 
/* Description:  Constructor                                                 */
/* Limitations:  None                                                        */
/* Return Value: None                                                        */
/*****************************************************************************/
GridSection::GridSection(QCanvas *canvas)
   : QCanvasRectangle( canvas )
{
   memset(TrackGrid, 0x00, sizeof(TrackGrid));
   TrackLevel = 0;
}

/*****************************************************************************/
/* Function:     GridSection                                                 */ 
/* Description:  Constructor                                                 */
/* Limitations:  None                                                        */
/* Return Value: None                                                        */
/*****************************************************************************/
GridSection::GridSection( QImage img, QCanvas *canvas )
   : QCanvasRectangle( canvas )
{
   memset(TrackGrid, 0x00, sizeof(TrackGrid));
   TrackImage = new QImage(img);

   TrackLevel = 0;

   setSize( TrackImage->width(), TrackImage->height() );

#ifndef _WS_QWS_
   TrackPixmap.convertFromImage(*TrackImage, OrderedAlphaDither);
#endif
}

/*****************************************************************************/
/* Function:     UpdateSpriteLocation                                        */ 
/* Description:                                                              */
/* Limitations:  None                                                        */
/* Return Value: None                                                        */
/*****************************************************************************/
void GridSection::UpdateSpriteLocation(int XLocation, int YLocation)
{
   SpriteXLocation = XLocation;
   SpriteYLocation = YLocation;
}

/*****************************************************************************/
/* Function:     LoadImage                                                   */ 
/* Description:                                                              */
/* Limitations:  None                                                        */
/* Return Value: None                                                        */
/*****************************************************************************/
void GridSection::LoadImage( TrackPieces ImageID, direction Direction,
                             int XCoord,  int YCoord, int ZCoord)
{
//   QImage *img;
   switch (ImageID)
   {
      case GRASS:
         TrackImage = new QImage(Grass);
         break;

      case VERT:
         TrackImage = new QImage(VerticalTrack);
         break;

      case START_TRACK:
         TrackImage = new QImage(VerticalStart);
         break;
         
      case VERT_BRIDGE:
         TrackImage = new QImage(VerticalBridge);
         break;

      case START_TRACK_BRIDGE:
         TrackImage = new QImage(VerticalStartBridge);
         break;

      case HORIZ:
         TrackImage = new QImage(HorizontalTrack);
         break;

      case HORIZ_START:
         TrackImage = new QImage(HorizontalStart);
         break;

      case HORIZ_BRIDGE:
         TrackImage = new QImage(HorizontalBridge);
         break;

      case LEFT_UPPER:
         TrackImage = new QImage(UpperLeft);
         break;

      case LEFT_UPPER_BRIDGE:
         TrackImage = new QImage(UpperLeft);
         break;

      case RIGHT_LOWER:
         TrackImage = new QImage(LowerRight);
         break;

      case RIGHT_LOWER_BRIDGE:
         TrackImage = new QImage(LowerRight);
         break;

      case RIGHT_UPPER:
      case RIGHT_UPPER_BRIDGE:
         TrackImage = new QImage(UpperRight);
         break;

      case RIGHT_UPPER_BRIDGE_START:
         TrackImage = new QImage(UpperRightStart);
         break;

      case LEFT_LOWER:
      case LEFT_LOWER_BRIDGE:
         TrackImage = new QImage(LowerLeft);
         break;

      default:
         break;
   }
   setSize( TrackImage->width(), TrackImage->height() );

   LoadGrid(ImageID);

   // These values are set relative to entire canvas not the
   // Min X and Y for individual image
   Min_XImageValue = XCoord * TrackImage->width();
   Min_YImageValue = YCoord * TrackImage->height();

   // These values are set relative to entire canvas not the
   // Max X and Y for individual image
   Max_XImageValue = ((XCoord + 1) * TrackImage->width())  - 1;
   Max_YImageValue = ((YCoord + 1) * TrackImage->height()) - 1;

   TrackImageID   = ImageID;
   TrackDirection = Direction;
   TrackLevel     = ZCoord * 2;  // Track Level should be either 0 or 2

#ifndef _WS_QWS_
   TrackPixmap.convertFromImage(*TrackImage, OrderedAlphaDither);
#endif
}

/*****************************************************************************/
/* Function:     SetImage                                                    */ 
/* Description:                                                              */
/* Limitations:  None                                                        */
/* Return Value: None                                                        */
/*****************************************************************************/
void GridSection::SetImage()
{
   move(Min_XImageValue, Min_YImageValue);
   setZ(TrackLevel);
   show();
}

/*****************************************************************************/
/* Function:     GetTrackDirection                                           */ 
/* Description:                                                              */
/* Limitations:  None                                                        */
/* Return Value: None                                                        */
/*****************************************************************************/
direction GridSection::GetTrackDirection(int GridValue)
{
   switch (TrackImageID)
   {
      case VERT:
      case VERT_BRIDGE:
      case START_TRACK:
      case START_TRACK_BRIDGE:
         return TrackDirection;
         break;

      case HORIZ:
      case HORIZ_BRIDGE:
      case HORIZ_START:
         return TrackDirection;
         break;

      case LEFT_UPPER:
      case LEFT_UPPER_BRIDGE:
         switch (GridValue)
         {
            case 1:
               if (TrackDirection == LEFT)
               {
                  return DOWN;
               }
               else
               {
                  return UP;
               }
               break;

            case 2:
            case 3:
               return TrackDirection;
               break;
         }
         break;

      case LEFT_LOWER:
      case LEFT_LOWER_BRIDGE:
         switch (GridValue)
         {
            case 1:
               if (TrackDirection == LEFT)
               {
                  return DOWN;
               }
               else
               {
                  return UP;
               }
               break;

            case 2:
               if (TrackDirection == LEFT)
               {
                  return RIGHT;
               }
               else
               {
                  return LEFT;
               }

            case 3:
               return TrackDirection;
               break;
         }
         break;

      case RIGHT_UPPER:
      case RIGHT_UPPER_BRIDGE:
      case RIGHT_UPPER_BRIDGE_START:
         switch (GridValue)
         {
            case 1:
               if (TrackDirection == LEFT)
               {
                  return UP;
               }
               else
               {
                  return DOWN;
               }
               break;

            case 2:
            case 3:
               return TrackDirection;
               break;
         }
         break;

      case RIGHT_LOWER:
      case RIGHT_LOWER_BRIDGE:
         switch (GridValue)
         {
            case 1:
               if (TrackDirection == LEFT)
               {
                  return UP;
               }
               else
               {
                  return DOWN;
               }
               break;

            case 2:
               if (TrackDirection == LEFT)
               {
                  return RIGHT;
               }
               else
               {
                  return LEFT;
               }
               break;

            case 3:
               return TrackDirection;
               break;
         }
         break;

      default:
         return TrackDirection;
         break;
   }
   return TrackDirection;
}


/*****************************************************************************/
/* Function:     GetGridXYValues                                             */ 
/* Description:                                                              */
/* Limitations:  None                                                        */
/* Return Value: None                                                        */
/*****************************************************************************/
void GridSection::GetGridXYValues(double ScreenX, double ScreenY, int *GridX, int *GridY)
{
   *GridX = (int)ScreenX - Min_XImageValue;
   *GridY = (int)ScreenY - Min_YImageValue;

   if ( (*GridX == 0) || (*GridY == 0) )
   {
      if (TrackGrid[*GridY][*GridX] == 1)
      {
         *GridY = *GridY + 3;
      }
      else if (TrackGrid[*GridY][*GridX] == 2)
      {
         *GridX = *GridX + 3;
      }
      else
      {
         *GridX = *GridX + 3;
         *GridY = *GridY + 3;
      }
   }
}

/*****************************************************************************/
/* Function:     OnGrid                                                      */ 
/* Description:                                                              */
/* Limitations:  None                                                        */
/* Return Value: None                                                        */
/*****************************************************************************/
int GridSection::OnGrid(int XLocation, int YLocation, int ZLocation)
{
   bool found = false;
   if ( (XLocation >= Min_XImageValue) && (XLocation <= Max_XImageValue) )
   {
      if ( (YLocation >= Min_YImageValue) &&
           (YLocation <= Max_YImageValue) )
      {
         if (ZLocation == (TrackLevel+1))
         {
            found = true;
         }
      }
      else
      {
         found = false;
      }
   }

   if (found)
   {
      return 0;
   }
   else
   {
      return 1;
   }
}

/*****************************************************************************/
/* Function:     LoadGrid                                                    */ 
/* Description:                                                              */
/* Limitations:  None                                                        */
/* Return Value: None                                                        */
/*****************************************************************************/
void GridSection::LoadGrid(int ImageID)
{
   switch (ImageID)
   {
      case VERT:
      case START_TRACK:
#ifndef __BIG_GRID
         memcpy(TrackGrid, VerticalScaled2, sizeof(TrackGrid));
#else
         memcpy(TrackGrid, LargeVerticalTrack2, sizeof(TrackGrid));
#endif
         break;

      case VERT_BRIDGE:
      case START_TRACK_BRIDGE:
         memcpy(TrackGrid, VerticalBridge2, sizeof(TrackGrid));
         break;

      case HORIZ:
      case HORIZ_START:
#ifndef __BIG_GRID
         memcpy(TrackGrid, HorizontalScaled2, sizeof(TrackGrid));
#else
         memcpy(TrackGrid, LargeHorizontalTrack2, sizeof(TrackGrid));
#endif
         break;

      case HORIZ_BRIDGE:
         memcpy(TrackGrid, HorizontalBridge2, sizeof(TrackGrid));
         break;

      case RIGHT_UPPER:
#ifndef __BIG_GRID
         memcpy(TrackGrid, UpperRightScaled2, sizeof(TrackGrid));
#else
         memcpy(TrackGrid, LargeUpperRightTrack2, sizeof(TrackGrid));
#endif
         break;

      case RIGHT_LOWER:
#ifndef __BIG_GRID
         memcpy(TrackGrid, LowerRightScaled2, sizeof(TrackGrid));
#else
         memcpy(TrackGrid, LargeLowerRightTrack2, sizeof(TrackGrid));
#endif
         break;

      case LEFT_UPPER:
#ifndef __BIG_GRID
         memcpy(TrackGrid, UpperLeftScaled2, sizeof(TrackGrid));
#else
         memcpy(TrackGrid, LargeUpperLeftTrack2, sizeof(TrackGrid));
#endif
         break;

      case LEFT_LOWER:
#ifndef __BIG_GRID
         memcpy(TrackGrid, LowerLeftScaled2, sizeof(TrackGrid));
#else
         memcpy(TrackGrid, LargeLowerLeftTrack2, sizeof(TrackGrid));
#endif
         break;

      case RIGHT_UPPER_BRIDGE:
      case RIGHT_UPPER_BRIDGE_START:
         memcpy(TrackGrid, UpperRightBridge2, sizeof(TrackGrid));
         break;

      case RIGHT_LOWER_BRIDGE:
         memcpy(TrackGrid, LowerRightBridge2, sizeof(TrackGrid));
         break;

      case LEFT_UPPER_BRIDGE:
         memcpy(TrackGrid, UpperLeftBridge2, sizeof(TrackGrid));
         break;

      case LEFT_LOWER_BRIDGE:
         memcpy(TrackGrid, LowerLeftBridge2, sizeof(TrackGrid));
         break;

      default:
#ifndef __BIG_GRID
         memcpy(TrackGrid, GrassArea, sizeof(TrackGrid));
#else
         memcpy(TrackGrid, LargeGrassArea2, sizeof(TrackGrid));
#endif
         break;
   }
}


/*****************************************************************************/
/* Function:     drawShape                                                   */ 
/* Description:                                                              */
/* Limitations:  None                                                        */
/* Return Value: None                                                        */
/*****************************************************************************/
void GridSection::drawShape( QPainter &p )
{
// On Qt/Embedded, we can paint a QImage as fast as a QPixmap,
// but on other platforms, we need to use a QPixmap.
#ifdef _WS_QWS_
   p.drawImage( int(x()), int(y()), *TrackImage, 0, 0, -1, -1, OrderedAlphaDither );
#else
   p.drawPixmap( int(x()), int(y()), TrackPixmap );
#endif
}
