/***************************************************************************
                           grid.h  -  description
                             -------------------
    begin                : Wed Aug 21 01:49:10 EDT 2002
    copyright            : (C) 2002 by Kevin Greenhaw
    email                : kevin_greenhaw@email.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef __GRID_H__
#define __GRID_H__

#include <qcanvas.h>
#include <qimage.h>
#include <qpixmap.h>

#include "track_image.h"

const int GRID_X = 75;
const int GRID_Y = 73;

static const int imageRTTI = 984376;

//********************************//
//******* Image Item Class *******//
class ImageItem: public QCanvasRectangle
{
public:
   ImageItem( QImage img, QCanvas *canvas );
   ImageItem( QPixmap img, QCanvas *canvas );
   ImageItem( QCanvas *canvas );
   
   void SetImage();
   void SetImage(int XCoord, int YCoord);

protected:
    void drawShape( QPainter & );
private:
   int Min_XImageValue;
   int Min_YImageValue;

   QImage image;
   QPixmap pixmap;
};


//**********************************//
//******* Grid Section Class *******//
class GridSection: public QCanvasRectangle
{
public:
   GridSection( QImage img, QCanvas *canvas );
   GridSection( QCanvas *canvas );

   void LoadImage( TrackPieces ImageID, direction Direction,
                   int XCoord,  int YCoord, int ZCoord);
   void SetImage();
   int  OnGrid(int XLocation, int YLocation, int ZLocation);
   void GetGridXYValues(double ScreenX, double ScreenY, int *GridX, int *GridY);
   
   direction GetTrackDirection(int GridValue);

   int GetGridMinXValue() { return Min_XImageValue; }
   int GetGridMinYValue() { return Min_YImageValue; }
   int GetGridMaxXValue() { return Max_XImageValue; }
   int GetGridMaxYValue() { return Max_YImageValue; }
   bool IsStartTrack()    { return ((TrackImageID == START_TRACK)        ||
                                    (TrackImageID == START_TRACK_BRIDGE) ||
                                    (TrackImageID == HORIZ_START)        ||
                                    (TrackImageID == RIGHT_UPPER_BRIDGE_START)); }
   void SetTrackValue(int value) { TrackValue = value; }
   int  GetTrackValue()   { return TrackValue; }
   
   void UpdateSpriteLocation(int XLocation, int YLocation);
   
   int rtti () const { return imageRTTI; }
   
   TrackPieces TrackImageID;
   int TrackGrid[GRID_Y][GRID_X];

protected:
    void drawShape( QPainter & );

private:
   QImage *TrackImage;
   QPixmap TrackPixmap;
   int TrackValue;
   int TrackLevel;

   direction TrackDirection;
   
   int Min_XImageValue;
   int Min_YImageValue;

   int Max_XImageValue;
   int Max_YImageValue;
   
   int SpriteXLocation;
   int SpriteYLocation;
   
   void LoadGrid(int ImageID);
};

#endif
