/***************************************************************************
                          racecar.h  -  description
                             -------------------
    begin                : Wed Aug 21 01:49:10 EDT 2002
    copyright            : (C) 2002 by Kevin Greenhaw
    email                : kevin_greenhaw@email.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#ifndef __RACE_CAR_H__
#define __RACE_CAR_H__

#include <qcanvas.h>
#include <qrect.h>

#include "grid.h"

const int MAX_SPEED      = 5;
const int TURN_RADIUS    = 30;
const int MAX_COLLISIONS = 1;
const int MAX_X_VALUE    = 225;
const int MAX_Y_VALUE    = 300;

static const int raceCarRTTI = 984999;

typedef enum carControlType
{
   COMPUTER_CONTROL = 0,
   USER_CONTROL = 1
} CarControlType;

//********************************//
//******** Car Line Class ********//
class CarLine: public QCanvasLine
{
public:
   CarLine( QCanvas *canvas );
   void SetPoints( QRect & );
};


class RaceCar : public QCanvasSprite
{
public:
   int TrackParts[16];
   
   RaceCar(QCanvasPixmapArray *s, QCanvas *c, CarControlType k);
   
   CarControlType GetCarControl() const {return CarControl;}

   int  GetRaceCarSpeed() const {return RaceCarSpeed;}
   int  GetRaceCarMaxSpeed() const { return RaceCarMaxSpeed; }
   void SetRaceCarSpeed(int Speed);
   void SetRaceCarMaxSpeed(int Speed) {RaceCarMaxSpeed = Speed;}

   int  GetRaceCarDirection() const {return RaceCarDirection;}
   int  GetRaceCarPicIndex() const {return RaceCarPictureIndex;}
   void SetRaceCarDirection(int Direction);

   int GetWidth() const { return width();  }
   int GetHeight() const { return height(); }

   void UpdateComputerControlCar(int LocalX, int LocalY, GridSection *trackPiece);
   
   void UpdateUserControlCar(int LocalX, int LocalY, GridSection *trackPiece);
   
   void UpdateLapsCompleted() { LapsCompleted += 1; }
   int  GetLapsCompleted() const { return LapsCompleted; }
   void SetRadiusAdjust(int Adjust) { RadiusAdjust = Adjust; }
   void ClearTurnDelay() { TurnDelay = FALSE; }

   int CheckForCollision();
   void ClearCollisionFlag() { CollisionFlag = 0; }

   void GetCarLocation(QRect *CarLocation);

   void IgnoreCollisions(bool flag) { CarCollisionsIgnored = flag; }

   void SetRestartFlag(bool flag) { RaceCarRestarted = flag; }
   bool GetRestartFlag() const { return RaceCarRestarted; }

   void SetRaceCarOffset(int offsetX, int offsetY);
   void RaceCar::ResetRace();

   QRect GetRaceCarCenterline();

   int rtti () const { return raceCarRTTI; }

   int RestartFrameCounter;

protected:
   static const double COS_15;
   static const double COS_30;
   static const double COS_45;
   static const double COS_60;
   static const double COS_75;
   static const double SIN_15;
   static const double SIN_30;
   static const double SIN_45;
   static const double SIN_60;
   static const double SIN_75;

private:
   int RaceCarDirection;
   int RaceCarPictureIndex;
   int RaceCarSpeed;
   int RaceCarMaxSpeed;
   int RaceCarTurnRadius;
   int ArcLength;
   int RadiusAdjust;
   int PreviousCarDirection;
   int LapsCompleted;
   
   int  TurnDelay;
   bool TurnComplete;
   bool RaceCarSpeedChange;
   
   int CollisionFlag;

   bool CarCollisionsIgnored;
   bool RaceCarRestarted;

   direction TurnDirection;
   CarControlType CarControl;

#if (QT_VERSION == 301)
   QPtrList<QPoint> TurnPoints;
#else
   QList<QPoint> TurnPoints;
#endif

   void DoAutoMove(int LocalX, int LocalY, 
                   GridSection *trackPiece, QCanvasText *TextBox);
   
   void GetTurnDimensions(int SideA, int SideB, float *Radius, int *ArcLength);
   
   void CreateTurnPointArray(int CurrentX, int CurrentY,
                             int Direction, direction TrackDirection);
   
   void CreateTurnPointArray2(int CurrentX, int CurrentY,
                              int Direction, direction TrackDirection);

   void GetTurnIncrement(float *DeltaX, float *DeltaY);

   int GetNewCarDirection(int       CarDirection,
                          int       GridDirection,
                          direction TurnDirection);

   int GetNewPictureIndex();

   void SetVelocity(int CarDirection);

   void UpdateCarZLevel(int GridDirection);

   bool UpdateCarDirection(int       CarDirection,
                           int       GridDirection,
                           direction TrackDirection1,
                           direction TrackDirection2,
                           direction TrackDirection3);

   int  UpdateRaceCarDirection(int TurnRadius, direction TrackDirection,
                               int MinPixIndex, int MaxPixIndex);
};

#endif
