 /***************************************************************************
                          startscreen.cpp  -  description
                             -------------------
    begin                : Wed Aug 21 01:49:10 EDT 2002
    copyright            : (C) 2002 by Kevin Greenhaw
    email                : kevin_greenhaw@email.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qfont.h>
#include <qglobal.h>
#include <qkeycode.h>

#include "track_image.h"
#include "startscreen.h"
#include "zrally.h"
#include "zrally_images.h"

const int LINE1 = 150;
const int LINE2 = 175;
const int LINE3 = 210;
const int LINE4 = 245;

/*****************************************************************************/
/* Function:     StartScreen                                                 */
/* Description:  Constructor                                                 */
/* Limitations:  None                                                        */
/* Return Value: None                                                        */
/*****************************************************************************/
StartScreen::StartScreen(QCanvas& c, QWidget* parent, const char* name)
   : QWidget( parent, name ),
     canvas(c)
{
   SelectedLabel = TRACK_SELECT;
   CurrentLevel  = LEVEL_1;
   NumberOfLaps  = 5;
   TrackDisplay  = 0;

   TrackList.setAutoDelete(true);

   editor = new QCanvasView(&canvas, this);
   editor->setGeometry(8, 0, 225, 292);
   editor->setVScrollBarMode(QScrollView::AlwaysOff);
   editor->setHScrollBarMode(QScrollView::AlwaysOff);
   editor->setFrameStyle( QFrame::NoFrame );

   AcceptKeyPressEvent = true;
}

/*****************************************************************************/
/* Function:     LoadTrackImages                                             */
/* Description:  Load all available Track Images into a list.                */
/* Limitations:  None                                                        */
/* Return Value: None                                                        */
/*****************************************************************************/
void StartScreen::Initialize()
{
   ShowTrack(TrackDisplay);
   DisplayOnScreenMenu();
}

/*****************************************************************************/
/* Function:     Hide                                                        */
/* Description:                                                              */
/* Limitations:  None                                                        */
/* Return Value: None                                                        */
/*****************************************************************************/
void StartScreen::Hide()
{
   delete LeftSelect;
   delete RightSelect;
   delete TitleImage;
   delete TrackSelectLabel;
   delete LevelSelectLable;
   delete SelectedLevel;
   delete LapsSelectLabel;
   delete SelectedLaps;
   delete StartRaceLabel;

   QCanvasPolygonalItem *trackPiece = zrally->GetTrackImage(TrackDisplay);
   trackPiece->hide();

   CurrentLevel  = LEVEL_1;
   SelectedLabel = TRACK_SELECT;
   NumberOfLaps  = 5;

   hide();
}

/*****************************************************************************/
/* Function:     Show                                                        */
/* Description:                                                              */
/* Limitations:  None                                                        */
/* Return Value: None                                                        */
/*****************************************************************************/
void StartScreen::Show()
{
   Initialize();
   show();
}

/*****************************************************************************/
/* Function:     ShowTrack                                                   */
/* Description:                                                              */
/* Limitations:  None                                                        */
/* Return Value: None                                                        */
/*****************************************************************************/
void StartScreen::ShowTrack(int Image)
{
   static QCanvasPolygonalItem *trackPiece = NULL;
   
   if (trackPiece != NULL)
   {
      trackPiece->hide();
   }

   trackPiece = zrally->GetTrackImage(Image);
   trackPiece->show();

   TrackDisplay = Image;
}

/*****************************************************************************/
/* Function:     DisplayOnScreenMenu                                         */
/* Description:                                                              */
/* Limitations:  None                                                        */
/* Return Value: None                                                        */
/*****************************************************************************/
void StartScreen::DisplayOnScreenMenu()
{
   QRect textRect;

   QColor textColor(75, 234, 254);
   QColor selectColor(0xFF, 0xFF, 0x50);

   QFont selectFont("Helvetica", 12, QFont::Bold, false);
   QFont textFont("Helvetica", 10, QFont::Normal, false);

   // Create the Left Direction Arrows, have them initially hid because there
   // are no choices available for left key pressed.
   LeftSelect  = new LevelSelectBox(&canvas, "<<", selectColor);
   LeftSelect->setFont(selectFont);
   LeftSelect->setX(52);
   LeftSelect->setY(LINE1);
   LeftSelect->setZ(1);
   LeftSelect->hide();

   // Create the Right Direction Arrows
   RightSelect = new LevelSelectBox(&canvas, ">>", selectColor);
   RightSelect->setFont(selectFont);
   RightSelect->setX(166);
   RightSelect->setY(LINE1);
   RightSelect->setZ(1);
   RightSelect->show();

   // Load the ZRally Logo
   TitleImage = new ImageItem(zrallyImages->GetImage(ZRallyImages::LOGO), &canvas);

   textRect = TitleImage->boundingRect();
   TitleImage->setX( (canvas.width()/2) - (textRect.width()/2) );
   TitleImage->setY(5);
   TitleImage->setZ(1);
   TitleImage->show();

   TrackSelectLabel = new LevelSelectBox(&canvas, "Select Track", selectColor);
   TrackSelectLabel->setFont(selectFont);

   textRect = TrackSelectLabel->boundingRect();

   TrackSelectLabel->setX( (canvas.width()/2) - (textRect.width()/2) );
   TrackSelectLabel->setY(LINE1);
   TrackSelectLabel->setZ(1);
   TrackSelectLabel->show();

   LevelSelectLable = new LevelSelectBox(&canvas, "Select Level", textColor);
   LevelSelectLable->setFont(textFont);

   textRect = LevelSelectLable->boundingRect();

   LevelSelectLable->setX( (canvas.width()/2) - (textRect.width()/2) );
   LevelSelectLable->setY(LINE2);
   LevelSelectLable->setZ(1);
   LevelSelectLable->show();

   SelectedLevel = new LevelSelectBox(&canvas, "Easy", textColor);
   SelectedLevel->setFont(textFont);

   textRect = SelectedLevel->boundingRect();
   SelectedLevel->setX( (canvas.width()/2) - (textRect.width()/2) );
   SelectedLevel->setY(LINE2+15);
   SelectedLevel->setZ(1);
   SelectedLevel->show();

   LapsSelectLabel = new LevelSelectBox(&canvas, "Laps", textColor);
   LapsSelectLabel->setFont(textFont);

   textRect = LapsSelectLabel->boundingRect();

   LapsSelectLabel->setX( (canvas.width()/2) - (textRect.width()/2) );
   LapsSelectLabel->setY(LINE3);
   LapsSelectLabel->setZ(1);
   LapsSelectLabel->show();

   SelectedLaps = new LevelSelectBox(&canvas, "5", textColor);
   SelectedLaps->setFont(textFont);

   textRect = SelectedLaps->boundingRect();
   SelectedLaps->setX( (canvas.width()/2) - (textRect.width()/2) );
   SelectedLaps->setY(LINE3+15);
   SelectedLaps->setZ(1);
   SelectedLaps->show();

   StartRaceLabel = new LevelSelectBox(&canvas, "Start Race", textColor);
   StartRaceLabel->setFont(textFont);

   textRect = StartRaceLabel->boundingRect();

   StartRaceLabel->setX( (canvas.width()/2) - (textRect.width()/2) );
   StartRaceLabel->setY(LINE4);
   StartRaceLabel->setZ(1);
   StartRaceLabel->show();
}


/*****************************************************************************/
/* Function:     ChangeSelectedLevel                                         */
/* Description:                                                              */
/* Limitations:  None                                                        */
/* Return Value: None                                                        */
/*****************************************************************************/
void StartScreen::ChangeSelectedLevel(int Direction)
{
   QColor textColor(75, 234, 254);
   QColor selectColor(0xFF, 0xFF, 0x50);

   QFont selectFont("Helvetica", 12, QFont::Bold, false);
   QFont textFont("Helvetica", 10, QFont::Normal, false);

   QRect textRect;

   switch (Direction)
   {
      case Key_Up:
         switch ( SelectedLabel )
         {
            case LEVEL_SELECT:
               TrackSelectLabel->setColor(selectColor);
               TrackSelectLabel->setFont(selectFont);

               textRect = TrackSelectLabel->boundingRect();
               TrackSelectLabel->setX( (canvas.width()/2) - (textRect.width()/2) );

               LevelSelectLable->setColor(textColor);
               LevelSelectLable->setFont(textFont);

               textRect = LevelSelectLable->boundingRect();
               LevelSelectLable->setX( (canvas.width()/2) - (textRect.width()/2) );

               SelectedLevel->setColor(textColor);
               SelectedLevel->setFont(textFont);

               textRect = SelectedLevel->boundingRect();
               SelectedLevel->setX( (canvas.width()/2) - (textRect.width()/2) );

               LeftSelect->setY(LINE1);
               RightSelect->setY(LINE1);

               if (TrackDisplay == 0)
               {
                  LeftSelect->hide();
                  RightSelect->show();
               }
               else if (TrackDisplay == MAX_TRACKS)
               {
                  LeftSelect->show();
                  RightSelect->hide();
               }
               else
               {
                  LeftSelect->show();
                  RightSelect->show();
               }

               emit RaceLevelUpdated((int)CurrentLevel);

               SelectedLabel = TRACK_SELECT;
               break;

            case LAPS_SELECT:
               LevelSelectLable->setColor(selectColor);
               LevelSelectLable->setFont(selectFont);

               textRect = LevelSelectLable->boundingRect();
               LevelSelectLable->setX( (canvas.width()/2) - (textRect.width()/2) );

               SelectedLevel->setColor(selectColor);
               SelectedLevel->setFont(selectFont);

               textRect = SelectedLevel->boundingRect();
               SelectedLevel->setX( (canvas.width()/2) - (textRect.width()/2) );

               LapsSelectLabel->setColor(textColor);
               LapsSelectLabel->setFont(textFont);

               textRect = LapsSelectLabel->boundingRect();
               LapsSelectLabel->setX( (canvas.width()/2) - (textRect.width()/2) );

               SelectedLaps->setColor(textColor);
               SelectedLaps->setFont(textFont);

               textRect = SelectedLaps->boundingRect();
               SelectedLaps->setX( (canvas.width()/2) - (textRect.width()/2) );

               LeftSelect->setY(LINE2);
               RightSelect->setY(LINE2);


               if (CurrentLevel == LEVEL_1)
               {
                  LeftSelect->hide();
                  RightSelect->show();
               }
               else if (CurrentLevel == LEVEL_3)
               {
                  LeftSelect->show();
                  RightSelect->hide();
               }
               else
               {
                  LeftSelect->show();
                  RightSelect->show();
               }

               emit RaceLengthUpdated(NumberOfLaps);

               SelectedLabel = LEVEL_SELECT;
               break;

            case START_RACE:
               LapsSelectLabel->setColor(selectColor);
               LapsSelectLabel->setFont(selectFont);

               textRect = LapsSelectLabel->boundingRect();
               LapsSelectLabel->setX( (canvas.width()/2) - (textRect.width()/2) );

               SelectedLaps->setColor(selectColor);
               SelectedLaps->setFont(selectFont);

               textRect = SelectedLaps->boundingRect();
               SelectedLaps->setX( (canvas.width()/2) - (textRect.width()/2) );

               StartRaceLabel->setColor(textColor);
               StartRaceLabel->setFont(textFont);

               textRect = StartRaceLabel->boundingRect();
               StartRaceLabel->setX( (canvas.width()/2) - (textRect.width()/2) );

               LeftSelect->setY(LINE3);
               RightSelect->setY(LINE3);

               if (NumberOfLaps == 5)
               {
                  LeftSelect->hide();
                  RightSelect->show();
               }
               else if (NumberOfLaps == 25)
               {
                  LeftSelect->show();
                  RightSelect->hide();
               }
               else
               {
                  LeftSelect->show();
                  RightSelect->show();
               }

               SelectedLabel = LAPS_SELECT;
               break;

            default:
               SelectedLabel = TRACK_SELECT;
               break;
         }
         break;

      case Key_Down:
         switch ( SelectedLabel )
         {
            case TRACK_SELECT:
               TrackSelectLabel->setColor(textColor);
               TrackSelectLabel->setFont(textFont);

               textRect = TrackSelectLabel->boundingRect();
               TrackSelectLabel->setX( (canvas.width()/2) - (textRect.width()/2) );

               LevelSelectLable->setColor(selectColor);
               LevelSelectLable->setFont(selectFont);

               textRect = LevelSelectLable->boundingRect();
               LevelSelectLable->setX( (canvas.width()/2) - (textRect.width()/2) );

               SelectedLevel->setColor(selectColor);
               SelectedLevel->setFont(selectFont);

               textRect = SelectedLevel->boundingRect();
               SelectedLevel->setX( (canvas.width()/2) - (textRect.width()/2) );

               LeftSelect->setY(LINE2);
               RightSelect->setY(LINE2);

               if (CurrentLevel == LEVEL_1)
               {
                  LeftSelect->hide();
                  RightSelect->show();
               }
               else if (CurrentLevel == LEVEL_3)
               {
                  LeftSelect->show();
                  RightSelect->hide();
               }
               else
               {
                  LeftSelect->show();
                  RightSelect->show();
               }

               emit RaceTrackUpdated(TrackDisplay);

               SelectedLabel = LEVEL_SELECT;
               break;

            case LEVEL_SELECT:
               LevelSelectLable->setColor(textColor);
               LevelSelectLable->setFont(textFont);

               textRect = LevelSelectLable->boundingRect();
               LevelSelectLable->setX( (canvas.width()/2) - (textRect.width()/2) );

               SelectedLevel->setColor(textColor);
               SelectedLevel->setFont(textFont);

               textRect = SelectedLevel->boundingRect();
               SelectedLevel->setX( (canvas.width()/2) - (textRect.width()/2) );

               LapsSelectLabel->setColor(selectColor);
               LapsSelectLabel->setFont(selectFont);

               textRect = LapsSelectLabel->boundingRect();
               LapsSelectLabel->setX( (canvas.width()/2) - (textRect.width()/2) );

               SelectedLaps->setColor(selectColor);
               SelectedLaps->setFont(selectFont);

               textRect = SelectedLevel->boundingRect();
               SelectedLevel->setX( (canvas.width()/2) - (textRect.width()/2) );

               LeftSelect->setY(LINE3);
               RightSelect->setY(LINE3);

               if (NumberOfLaps == 5)
               {
                  LeftSelect->hide();
                  RightSelect->show();
               }
               else if (NumberOfLaps == 25)
               {
                  LeftSelect->show();
                  RightSelect->hide();
               }
               else
               {
                  LeftSelect->show();
                  RightSelect->show();
               }

               emit RaceLevelUpdated((int)CurrentLevel);

               SelectedLabel = LAPS_SELECT;
               break;

            case LAPS_SELECT:
               LapsSelectLabel->setColor(textColor);
               LapsSelectLabel->setFont(textFont);

               textRect = LapsSelectLabel->boundingRect();
               LapsSelectLabel->setX( (canvas.width()/2) - (textRect.width()/2) );

               SelectedLaps->setColor(textColor);
               SelectedLaps->setFont(textFont);

               textRect = SelectedLevel->boundingRect();
               SelectedLevel->setX( (canvas.width()/2) - (textRect.width()/2) );

               StartRaceLabel->setColor(selectColor);
               StartRaceLabel->setFont(selectFont);

               textRect = StartRaceLabel->boundingRect();
               StartRaceLabel->setX( (canvas.width()/2) - (textRect.width()/2) );

               LeftSelect->setY(LINE4);
               RightSelect->setY(LINE4);

               LeftSelect->show();
               RightSelect->show();

               emit RaceLengthUpdated(NumberOfLaps);

               SelectedLabel = START_RACE;
               break;

            default:
               SelectedLabel = START_RACE;
               break;
         }
         break;

      case Key_Left:
         switch( SelectedLabel )
         {
            case TRACK_SELECT:
               TitleImage->hide();

               if (TrackDisplay > 0)
               {
                  ShowTrack(TrackDisplay-1);
                  RightSelect->show();
               }
               if (TrackDisplay == 0)
               {
                  LeftSelect->hide();
               }
               break;

            case LEVEL_SELECT:
               if (CurrentLevel == LEVEL_2)
               {
                  SelectedLevel->setText("Easy");
                  CurrentLevel = LEVEL_1;
                  LeftSelect->hide();
               }
               else if (CurrentLevel == LEVEL_3)
               {
                  SelectedLevel->setText("Medium");
                  CurrentLevel = LEVEL_2;
                  RightSelect->show();
               }
               else
               {
                  LeftSelect->hide();
               }
               textRect = SelectedLevel->boundingRect();
               SelectedLevel->setX( (canvas.width()/2) - (textRect.width()/2) );
               break;

            case LAPS_SELECT:
               if (NumberOfLaps == 25)
               {
                  RightSelect->show();
                  SelectedLaps->setText("20");
                  NumberOfLaps = 20;
               }
               else if (NumberOfLaps == 20)
               {
                  SelectedLaps->setText("15");
                  NumberOfLaps = 15;
               }
               else if (NumberOfLaps == 15)
               {
                  SelectedLaps->setText("10");
                  NumberOfLaps = 10;
               }
               else if (NumberOfLaps == 10)
               {
                  SelectedLaps->setText("5");
                  NumberOfLaps = 5;
                  LeftSelect->hide();
               }
               else
               {
                  SelectedLaps->setText("1");
                  NumberOfLaps = 1;
               }
               textRect = SelectedLaps->boundingRect();
               SelectedLaps->setX( (canvas.width()/2) - (textRect.width()/2) );
               break;

            case START_RACE:
               emit StartTheRace();
               break;
         }
         break;

      case Key_Right:
         switch( SelectedLabel )
         {
            case TRACK_SELECT:
               TitleImage->hide();

               if (TrackDisplay < MAX_TRACKS)
               {
                  ShowTrack(TrackDisplay+1);
                  LeftSelect->show();
               }
               if (TrackDisplay == MAX_TRACKS)
               {
                  RightSelect->hide();
               }
               break;

            case LEVEL_SELECT:
               if (CurrentLevel == LEVEL_1)
               {
                  SelectedLevel->setText("Medium");
                  CurrentLevel = LEVEL_2;
                  LeftSelect->show();
               }
               else if (CurrentLevel == LEVEL_2)
               {
                  SelectedLevel->setText("Hard");
                  CurrentLevel = LEVEL_3;
                  RightSelect->hide();
               }
               else
               {
                  RightSelect->hide();
               }
               textRect = SelectedLevel->boundingRect();
               SelectedLevel->setX( (canvas.width()/2) - (textRect.width()/2) );
               break;

            case LAPS_SELECT:
               if (NumberOfLaps == 1)
               {
                  SelectedLaps->setText("5");
                  NumberOfLaps = 5;
               }
               else if (NumberOfLaps == 5)
               {
                  SelectedLaps->setText("10");
                  NumberOfLaps = 10;
                  LeftSelect->show();
               }
               else if (NumberOfLaps == 10)
               {
                  SelectedLaps->setText("15");
                  NumberOfLaps = 15;
               }
               else if (NumberOfLaps == 15)
               {
                  SelectedLaps->setText("20");
                  NumberOfLaps = 20;
               }
               else
               {
                  SelectedLaps->setText("25");
                  NumberOfLaps = 25;
                  RightSelect->hide();
               }
               textRect = SelectedLaps->boundingRect();
               SelectedLaps->setX( (canvas.width()/2) - (textRect.width()/2) );
               break;

            case START_RACE:
               emit StartTheRace();
               break;
         }
         break;

      case Key_Escape:
      case Key_Return:
      case Key_Enter:
         if ( SelectedLabel == START_RACE)
         {
            emit StartTheRace();
         }
         break;
   }
   canvas.update();
}
