/***************************************************************************
                          startscreen.h  -  description
                             -------------------
    begin                : Wed Aug 21 01:49:10 EDT 2002
    copyright            : (C) 2002 by Kevin Greenhaw
    email                : kevin_greenhaw@email.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#ifndef __START_SCREEN_H__
#define __START_SCREEN_H__

#include <qwidget.h>
#include <qintdict.h>
#include <qcanvas.h>
#include <qmap.h>
#include <qimage.h>
#include <qpainter.h>
#include <qpushbutton.h>

#include <qglobal.h>

//#if (QT_VERSION == 301)
//   #include <qptrlist.h>
//#else
   #include <qlist.h>
//#endif

#include "grid.h"

typedef enum playLevel
{
   LEVEL_1,
   LEVEL_2,
   LEVEL_3
} PlayLevel;


class LevelSelectBox : public QCanvasText
{
   public:
      LevelSelectBox(QCanvas *c, QString textString, QColor textColor) : QCanvasText(c)
      {
         setText(textString);
         setColor(textColor);

         LevelText = textString;
      }

      void SetSelected(bool s){ Selected = s; }
      bool GetSelected(){ return Selected; }
      void redrawText(QPainter &p)
      {
         p.drawText( int(x()), int(y()), LevelText);
      }

   private:
      bool    Selected;
      QString LevelText;
};


class StartScreen : public QWidget
{
    Q_OBJECT

public:
   StartScreen(QCanvas& c, QWidget *parent = 0, const char *name = 0);
   void Initialize();
   void Hide();
   void Show();
   void ChangeSelectedLevel(int Direction);

public slots:

signals:
   void StartTheRace();
   void RaceLevelUpdated(int);
   void RaceLengthUpdated(int);
   void RaceTrackUpdated(int);

private:
   QCanvas&    canvas;
   QCanvasView *editor;

   PlayLevel   CurrentLevel;

   LevelSelectBox *LeftSelect;
   LevelSelectBox *RightSelect;

   LevelSelectBox       *StartRaceLabel;
   LevelSelectBox       *LapsSelectLabel;
   LevelSelectBox       *TrackSelectLabel;
   LevelSelectBox       *LevelSelectLable;
   LevelSelectBox       *SelectedLevel;
   LevelSelectBox       *SelectedLaps;

   QCanvasPolygonalItem *TitleImage;

   typedef enum labels
   {
      TRACK_SELECT,
      LEVEL_SELECT,
      LAPS_SELECT,
      START_RACE
   } Labels;

   Labels SelectedLabel;

   int NumberOfLaps;
   int TrackDisplay;

   bool AcceptKeyPressEvent;

   void LoadTrackImages();
   void DisplayOnScreenMenu();

//#if (QT_VERSION == 301)
//   QPtrList<QCanvasPolygonalItem> TrackList;
//#else
   QList<QCanvasPolygonalItem> TrackList;
//#endif

   void ShowTrack(int);
};

#endif
