/***************************************************************************
                         user_settings.cpp  -  description
                             -------------------
    begin                : Wed Aug 21 01:49:10 EDT 2002
    copyright            : (C) 2002 by Kevin Greenhaw
    email                : kevin_greenhaw@email.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include <qstring.h>

#include "user_settings.h"

UserSettings *zRallyConfig;

/*****************************************************************************/
/* Function:     ZScreen                                                     */
/* Description:  Constructor                                                 */
/* Limitations:  None                                                        */
/* Return Value: None                                                        */
/*****************************************************************************/
UserSettings::UserSettings()
{
}

/*****************************************************************************/
/* Function:     ReadFastestLaps                                             */
/* Description:  Reads the stored Fastest Lap values.                         */
/* Limitations:  None                                                        */
/* Return Value: None                                                        */
/*****************************************************************************/
void UserSettings::ReadFastestRaceTime()
{
#ifndef Z_QT
   Config zConfig("ZRally");
   zConfig.setGroup("zrally");
/* From Froot ---
   Config *config = new Config("FrootApp");
   for(int i=0; i<4; i++)
   m_highscore[i]->readScores(config);
   delete config;
*/
#else
   #if QT_VERSION > 233
      QSettings zConfig;
      zConfig.setPath(".zrally", "high_scores");
   #endif
#endif
   const QString key = "/LapTime/";
   
   QString trackIndex;
   QString data1,
           data2,
           data3,
           data4,
           data5;
   int loopIndex = 0;

   for (loopIndex = TRACK1; loopIndex <= TRACK15; loopIndex++)
   {
      trackIndex = "Track" + QString::number(loopIndex);
      
      data1 = zConfig.readEntry(key+trackIndex+"/FiveLaps", "999.99");
      data2 = zConfig.readEntry(key+trackIndex+"/TenLaps", "999.99");
      data3 = zConfig.readEntry(key+trackIndex+"/FifteenLaps", "999.99");
      data4 = zConfig.readEntry(key+trackIndex+"/TwentyLaps", "999.99");
      data5 = zConfig.readEntry(key+trackIndex+"/TwentyFiveLaps", "999.99");
      
      FiveLapRace.insert( loopIndex, data1.toDouble() );
      TenLapRace.insert( loopIndex, data2.toDouble() );
      FifteenLapRace.insert( loopIndex, data3.toDouble() );
      TwentyLapRace.insert( loopIndex, data4.toDouble() );
      TwentyFiveLapRace.insert( loopIndex, data5.toDouble() );
   }
}


/*****************************************************************************/
/* Function:     ReadFastestLaps                                             */
/* Description:  Reads the stored Fastest Lap values.                         */
/* Limitations:  None                                                        */
/* Return Value: None                                                        */
/*****************************************************************************/
void UserSettings::WriteFastestRaceTime()
{
#ifndef Z_QT
   Config zConfig("ZRally");
   zConfig.setGroup("zrally");
#else
   #if QT_VERSION > 233
      QSettings zConfig;
      zConfig.setPath(".zrally", "high_scores");
   #endif
#endif

   const QString key = "/LapTime/";
   QString trackIndex;
   int loopIndex = 0;

   for (loopIndex = TRACK1; loopIndex <= TRACK15; loopIndex++)
   {
      trackIndex = "Track"+QString::number(loopIndex);
      
      zConfig.writeEntry( key+trackIndex+"/FiveLaps", 
                          QString::number(FiveLapRace[loopIndex]) );
      zConfig.writeEntry( key+trackIndex+"/TenLaps", 
                          QString::number(TenLapRace[loopIndex]) );
      zConfig.writeEntry( key+trackIndex+"/FifteenLaps", 
                          QString::number(FifteenLapRace[loopIndex]) );
      zConfig.writeEntry( key+trackIndex+"/TwentyLaps", 
                          QString::number(TwentyLapRace[loopIndex]) );
      zConfig.writeEntry( key+trackIndex+"/TwentyFiveLaps", 
                          QString::number(TwentyFiveLapRace[loopIndex]) );
   }
}

void UserSettings::UpdateFastestRaceTime(int    TrackID, 
                                         int    Laps, 
                                         double RaceTime)
{
   switch (Laps)
   {
      case 5:
         FiveLapRace[TrackID] = RaceTime;
         break;
      
      case 10:
         TenLapRace[TrackID] = RaceTime;
         break;
         
      case 15:
         FifteenLapRace[TrackID] = RaceTime;
         break;
         
      case 20:
         TwentyLapRace[TrackID] = RaceTime;
         break;
         
      default:
         TwentyFiveLapRace[TrackID] = RaceTime;
         break;
   }
}
   
double UserSettings::FastestRaceTime(int TrackID, int Laps)
{
   double returnValue;
   switch (Laps)
   {
      case 5:
         returnValue = FiveLapRace[TrackID];
         break;
         
      case 10:
         returnValue = TenLapRace[TrackID];
         break;
         
      case 15:
         returnValue = FifteenLapRace[TrackID];
         break;
         
      case 20:
         returnValue = TwentyLapRace[TrackID];
         break;
         
      default:
         returnValue = TwentyFiveLapRace[TrackID];
         break;
   }
      
   return returnValue;
}

