/***************************************************************************
                          zrally.cpp  -  description
                             -------------------
    begin                : Wed Aug 21 01:49:10 EDT 2002
    copyright            : (C) 2002 by Kevin Greenhaw
    email                : kevin_greenhaw@email.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef Z_QT
   #include <qapplication.h>
   #include <qmenubar.h>
#endif

#include <qmessagebox.h>
#include <qdir.h>

#include "zrally.h"
#include "zscreen.h"
#include "startscreen.h"
#include "endscreen.h"
#include "zrally_images.h"
#include "user_settings.h"
#include "LogTrace.h"

using namespace log_functions;

ZRally *zrally;

/*****************************************************************************/
/* Function:     ZRally                                                      */ 
/* Description:  Constructor                                                 */
/* Limitations:  None                                                        */
/* Return Value: None                                                        */
/*****************************************************************************/
#ifdef Z_QT
ZRally::ZRally() : QMainWindow(0)
{
   QWidget *mainWin = new QWidget( this );
   mainWin->setFixedSize(240, 320);

   QMenuBar* menu = menuBar();

   QPopupMenu* file = new QPopupMenu;
   file->insertItem("E&xit", qApp, SLOT(quit()), CTRL+Key_Q);
   menu->insertItem("&File", file);
   menu->show();
   
#else
ZRally::ZRally(QWidget *parent, const char *name) 
   : QWidget(parent, name)
{
   showMaximized();
   setFixedWidth(240);
   
   QWidget *mainWin = this;
#endif

   QDir directory;

#ifdef _WIN32
   picsDirectory = "./";
#else
   picsDirectory = "/home/QtPalmtop/pics/zrally/";
#endif
   mainWin->setBackgroundColor(QColor (0x00, 0x80, 0x00));
   
   SetupComplete = false;

   AcceptKeyPressEvent = false;

   // Initialize the ZRally Images Class
   zrallyImages = new ZRallyImages();

   if (directory.exists(picsDirectory) == false)
   {
      QMessageBox::warning(this, "ZRally", 
                           "Could not locate Directory " + picsDirectory,
                           "Quit", 0, 0, 0, 1);
   }
   else if (LoadTrackImages() == true)
   {
      RaceStarted    = false;
      RaceIsOver     = false;
   
      RaceDifficulty = 0;
      NumberOfLaps   = 10;
      SelectedTrack  = 0;
   
      canvas.setDoubleBuffering(true);
      canvas.setBackgroundColor(QColor (0x00, 0x80, 0x00));
      canvas.resize(225, 292);

      logTrace = new LogTrace("ZRally");

      START_FUNCTION_TRACE("ZRally::ZRally");
      END_FUNCTION_TRACE("ZRally::ZRally");

      zRallyConfig = new UserSettings();
      zRallyConfig->ReadFastestRaceTime();

      startScreen = new StartScreen(canvas, mainWin);
      startScreen->setFocus();
      startScreen->resize(width(), height());
      startScreen->setBackgroundColor(QColor (0x00, 0x80, 0x00));
      startScreen->hide();
      connect(startScreen, SIGNAL(StartTheRace()), this, SLOT(RaceStart()));

      connect(startScreen, SIGNAL(RaceLevelUpdated(int)),
              this, SLOT(RaceLevel(int)));

      connect(startScreen, SIGNAL(RaceLengthUpdated(int)),
              this, SLOT(RaceLength(int)));

      connect(startScreen, SIGNAL(RaceTrackUpdated(int)),
              this, SLOT(RaceTrack(int)));

      zScreen = new ZScreen(canvas, mainWin);
      zScreen->setFocus();
      zScreen->resize(width(), height());
      zScreen->setBackgroundColor(QColor (0x00, 0x80, 0x00));
      zScreen->hide();
      connect(zScreen, SIGNAL(RaceOver(bool, long)),
              this, SLOT(RaceFinished(bool, long)));

      endScreen = new EndScreen(canvas, mainWin);
      endScreen->setFocus();
      endScreen->resize(width(), height());
      endScreen->setBackgroundColor(QColor (0x00, 0x80, 0x00));
      endScreen->hide();
      connect(endScreen, SIGNAL(ReturnToStart()), this, SLOT(ReturnToStart()));

#ifdef Z_QT
      setCaption("Z-Rally");
      setCentralWidget( mainWin );
      resize(240, 320);
#endif
      show();

      AcceptKeyPressEvent = true;
      SetupComplete = true;
   }
   else
   {
      QMessageBox::warning(this, "ZRally",
                           "Could not load Track Images",
                           "Quit", 0, 0, 0, 1);
   }
}

ZRally::~ZRally()
{
   delete zRallyConfig;
   delete startScreen;
   delete zScreen;
   delete endScreen;
   delete logTrace;
}

/*****************************************************************************/
/* Function:     LoadTrackImages                                             */
/* Description:  Load all available Track Images into a list.                */
/* Limitations:  None                                                        */
/* Return Value: None                                                        */
/*****************************************************************************/
bool ZRally::LoadTrackImages()
{
   QImage img;

   if (img.load(picsDirectory + "track_pics/TrackPix1.png") == false)
   {
      QMessageBox::warning(this, "ZRally",
                           "Could not load Track Images from " + picsDirectory,
                           "Quit", 0, 0, 0, 1);

      img = zrallyImages->GetImage(ZRallyImages::TRACK_NOT_AVAIL);
   }
   TrackList.append ( new ImageItem(img, &canvas) );
   TrackList.current()->setX(0);
   TrackList.current()->setY(0);
   TrackList.current()->setZ(0);

   if (img.load(picsDirectory + "track_pics/TrackPix2.png") == false)
   {
      QMessageBox::warning(this, "ZRally",
                           "Could not load Track Image TrackPix2.png",
                           "OK", 0, 0, 0, 1);
      img = zrallyImages->GetImage(ZRallyImages::TRACK_NOT_AVAIL);
   }
   TrackList.append ( new ImageItem(img, &canvas) );
   TrackList.current()->setX(0);
   TrackList.current()->setY(0);
   TrackList.current()->setZ(0);

   if (img.load(picsDirectory + "track_pics/TrackPix3.png") == false)
   {
      QMessageBox::warning(this, "ZRally",
                           "Could not load Track Image TrackPix3.png",
                           "OK", 0, 0, 0, 1);
      img = zrallyImages->GetImage(ZRallyImages::TRACK_NOT_AVAIL);
   }
   TrackList.append ( new ImageItem(img, &canvas) );
   TrackList.current()->setX(0);
   TrackList.current()->setY(0);
   TrackList.current()->setZ(0);

   if (img.load(picsDirectory + "track_pics/TrackPix4.png") == false)
   {
      QMessageBox::warning(this, "ZRally",
                           "Could not load Track Image TrackPix4.png",
                           "OK", 0, 0, 0, 1);
      img = zrallyImages->GetImage(ZRallyImages::TRACK_NOT_AVAIL);
   }
   TrackList.append ( new ImageItem(img, &canvas) );
   TrackList.current()->setX(0);
   TrackList.current()->setY(0);
   TrackList.current()->setZ(0);

   if (img.load(picsDirectory + "track_pics/TrackPix5.png") == false)
   {
      QMessageBox::warning(this, "ZRally",
                           "Could not load Track Image TrackPix5.png",
                           "OK", 0, 0, 0, 1);
      img = zrallyImages->GetImage(ZRallyImages::TRACK_NOT_AVAIL);
   }
   TrackList.append ( new ImageItem(img, &canvas) );
   TrackList.current()->setX(0);
   TrackList.current()->setY(0);
   TrackList.current()->setZ(0);

   if (img.load(picsDirectory + "track_pics/TrackPix6.png") == false)
   {
      QMessageBox::warning(this, "ZRally",
                           "Could not load Track Image TrackPix6.png",
                           "OK", 0, 0, 0, 1);
      img = zrallyImages->GetImage(ZRallyImages::TRACK_NOT_AVAIL);
   }
   TrackList.append ( new ImageItem(img, &canvas) );
   TrackList.current()->setX(0);
   TrackList.current()->setY(0);
   TrackList.current()->setZ(0);

   if (img.load(picsDirectory + "track_pics/TrackPix7.png") == false)
   {
      QMessageBox::warning(this, "ZRally",
                           "Could not load Track Image TrackPix7.png",
                           "OK", 0, 0, 0, 1);
      img = zrallyImages->GetImage(ZRallyImages::TRACK_NOT_AVAIL);
   }
   TrackList.append ( new ImageItem(img, &canvas) );
   TrackList.current()->setX(0);
   TrackList.current()->setY(0);
   TrackList.current()->setZ(0);

   if (img.load(picsDirectory + "track_pics/TrackPix8.png") == false)
   {
      QMessageBox::warning(this, "ZRally",
                           "Could not load Track Image TrackPix8.png",
                           "OK", 0, 0, 0, 1);
      img = zrallyImages->GetImage(ZRallyImages::TRACK_NOT_AVAIL);
   }
   TrackList.append ( new ImageItem(img, &canvas) );
   TrackList.current()->setX(0);
   TrackList.current()->setY(0);
   TrackList.current()->setZ(0);

   if (img.load(picsDirectory + "track_pics/TrackPix9.png") == false)
   {
      QMessageBox::warning(this, "ZRally",
                           "Could not load Track Image TrackPix9.png",
                           "OK", 0, 0, 0, 1);
      img = zrallyImages->GetImage(ZRallyImages::TRACK_NOT_AVAIL);
   }
   TrackList.append ( new ImageItem(img, &canvas) );
   TrackList.current()->setX(0);
   TrackList.current()->setY(0);
   TrackList.current()->setZ(0);

   if (img.load(picsDirectory + "track_pics/TrackPix10.png") == false)
   {
      QMessageBox::warning(this, "ZRally",
                           "Could not load Track Image TrackPix10.png",
                           "OK", 0, 0, 0, 1);
      img = zrallyImages->GetImage(ZRallyImages::TRACK_NOT_AVAIL);
   }
   TrackList.append ( new ImageItem(img, &canvas) );
   TrackList.current()->setX(0);
   TrackList.current()->setY(0);
   TrackList.current()->setZ(0);

   if (img.load(picsDirectory + "track_pics/TrackPix11.png") == false)
   {
      QMessageBox::warning(this, "ZRally",
                           "Could not load Track Image TrackPix11.png",
                           "OK", 0, 0, 0, 1);
      img = zrallyImages->GetImage(ZRallyImages::TRACK_NOT_AVAIL);
   }
   TrackList.append ( new ImageItem(img, &canvas) );
   TrackList.current()->setX(0);
   TrackList.current()->setY(0);
   TrackList.current()->setZ(0);

   if (img.load(picsDirectory + "track_pics/TrackPix12.png") == false)
   {
      QMessageBox::warning(this, "ZRally",
                           "Could not load Track Image TrackPix12.png",
                           "OK", 0, 0, 0, 1);
      img = zrallyImages->GetImage(ZRallyImages::TRACK_NOT_AVAIL);
   }
   TrackList.append ( new ImageItem(img, &canvas) );
   TrackList.current()->setX(0);
   TrackList.current()->setY(0);
   TrackList.current()->setZ(0);

   if (img.load(picsDirectory + "track_pics/TrackPix13.png") == false)
   {
      QMessageBox::warning(this, "ZRally",
                           "Could not load Track Image TrackPix13.png",
                           "OK", 0, 0, 0, 1);
      img = zrallyImages->GetImage(ZRallyImages::TRACK_NOT_AVAIL);
   }
   TrackList.append ( new ImageItem(img, &canvas) );
   TrackList.current()->setX(0);
   TrackList.current()->setY(0);
   TrackList.current()->setZ(0);

   if (img.load(picsDirectory + "track_pics/TrackPix14.png") == false)
   {
//      QMessageBox::warning(this, "ZRally",
//                           "Could not load Track Image TrackPix14.png",
//                           "OK", 0, 0, 0, 1);
      img = zrallyImages->GetImage(ZRallyImages::TRACK_NOT_AVAIL);
   }
   TrackList.append ( new ImageItem(img, &canvas) );
   TrackList.current()->setX(0);
   TrackList.current()->setY(0);
   TrackList.current()->setZ(0);

   if (img.load(picsDirectory + "track_pics/TrackPix15.png") == false)
   {
//      QMessageBox::warning(this, "ZRally",
//                           "Could not load Track Image TrackPix15.png",
//                           "OK", 0, 0, 0, 1);
      img = zrallyImages->GetImage(ZRallyImages::TRACK_NOT_AVAIL);
   }
   TrackList.append ( new ImageItem(img, &canvas) );
   TrackList.current()->setX(0);
   TrackList.current()->setY(0);
   TrackList.current()->setZ(0);

   return true;
}

/*****************************************************************************/
/* Function:     GetTrackImage                                               */
/* Description:  Load all available Track Images into a list.                */
/* Limitations:  None                                                        */
/* Return Value: None                                                        */
/*****************************************************************************/
QCanvasPolygonalItem *ZRally::GetTrackImage(int ImageIndex)
{
   return TrackList.at(ImageIndex);
}

/*****************************************************************************/
/* Function:     Start                                                       */
/* Description:  Load all available Track Images into a list.                */
/* Limitations:  None                                                        */
/* Return Value: None                                                        */
/*****************************************************************************/
void ZRally::Start()
{
//   startScreen->Initialize();
   if (SetupComplete)
   {
      startScreen->Show();
   }
   else
   {
//      QApplication::exit(1);
   }
}

/*****************************************************************************/
/* Function:     RaceLevel                                                   */
/* Description:  SLOT                                                        */
/* Limitations:  None                                                        */
/* Return Value: None                                                        */
/*****************************************************************************/
void ZRally::RaceLevel(int Level)
{
   RaceDifficulty = Level;
}

/*****************************************************************************/
/* Function:     RaceLength                                                  */
/* Description:  SLOT                                                        */
/* Limitations:  None                                                        */
/* Return Value: None                                                        */
/*****************************************************************************/
void ZRally::RaceLength(int Laps)
{
   NumberOfLaps = Laps;
}

/*****************************************************************************/
/* Function:     RaceTrack                                                   */
/* Description:  SLOT                                                        */
/* Limitations:  None                                                        */
/* Return Value: None                                                        */
/*****************************************************************************/
void ZRally::RaceTrack(int Track)
{
   SelectedTrack = Track;
}

/*****************************************************************************/
/* Function:     RaceStart                                                   */
/* Description:  SLOT                                                        */
/* Limitations:  None                                                        */
/* Return Value: None                                                        */
/*****************************************************************************/
void ZRally::RaceStart()
{

   RaceStarted = true;
   RaceIsOver  = false;
   startScreen->Hide();
   zScreen->show();

   zScreen->Initialize(SelectedTrack, RaceDifficulty, NumberOfLaps);
}

/*****************************************************************************/
/* Function:     RaceFinished                                                */
/* Description:  SLOT                                                        */
/* Limitations:  None                                                        */
/* Return Value: None                                                        */
/*****************************************************************************/
void ZRally::RaceFinished(bool Winner, long ElapsedTime)
{
   double raceRecord,
          averageTime,
          totalTime;
   
   RaceStarted = false;
   RaceIsOver  = true;
   zScreen->hide();
   
   totalTime   = static_cast<double>(ElapsedTime) / 1000.0; 
   raceRecord  = zRallyConfig->FastestRaceTime(SelectedTrack, NumberOfLaps);
   averageTime = static_cast<double>(totalTime / NumberOfLaps) ;
   
   if ( totalTime < raceRecord ) 
   {
      zRallyConfig->UpdateFastestRaceTime(SelectedTrack, 
                                          NumberOfLaps,
                                          totalTime);
      raceRecord = totalTime;
   }
   
   zRallyConfig->WriteFastestRaceTime();
   endScreen->Show(Winner, SelectedTrack, NumberOfLaps, totalTime, raceRecord);
}

/*****************************************************************************/
/* Function:     ReturnToStart                                                   */
/* Description:  SLOT                                                        */
/* Limitations:  None                                                        */
/* Return Value: None                                                        */
/*****************************************************************************/
void ZRally::ReturnToStart()
{
   RaceStarted = false;
   RaceIsOver  = false;
   zScreen->hide();
   endScreen->Hide();
   startScreen->Show();
}

/*****************************************************************************/
/* Function:     keyPressEvent                                               */
/* Description:  This function is called in response to a Key being pressed. */
/* Limitations:  None                                                        */
/* Return Value: None                                                        */
/*****************************************************************************/
void ZRally::keyPressEvent( QKeyEvent *k )
{
   if (RaceStarted)
   {
      if (k->isAutoRepeat() == false)
      {
         zScreen->keyPressEvent(k->key());
      }
   }
   else if (RaceIsOver)
   {
         endScreen->keyPressEvent();
   }
   else
   {
      startScreen->ChangeSelectedLevel(k->key());
   }
   k->accept();
}

/*****************************************************************************/
/* Function:     keyReleaseEvent                                             */
/* Description:  This function is called in response to a Key being pressed. */
/* Limitations:  None                                                        */
/* Return Value: None                                                        */
/*****************************************************************************/
void ZRally::keyReleaseEvent( QKeyEvent *k )
{
   if (RaceStarted)
   {
      if (k->isAutoRepeat() == false)
      {
         zScreen->keyReleaseEvent(k->key());
      }
   }
   k->accept();
}


/*****************************************************************************/
/* Function:     quit                                                        */
/* Description:  This function is called in response to a Key being pressed. */
/* Limitations:  None                                                        */
/* Return Value: None                                                        */
/*****************************************************************************/
void ZRally::quit()
{
//   QApplication::exit();
}
