/***************************************************************************
                          zscreen.cpp  -  description
                             -------------------
    begin                : Wed Aug 21 01:49:10 EDT 2002
    copyright            : (C) 2002 by Kevin Greenhaw
    email                : kevin_greenhaw@email.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qdatetime.h>
#include <qmessagebox.h>
#include <qpainter.h>
#include <qapplication.h>
#include <qlabel.h>
#include <qprogressdialog.h>
#include <qkeycode.h>
#include <qtimer.h>
#include <qstring.h>
#include <qnamespace.h>
#include "zrally.h"
#include "zscreen.h"
#include "track_image.h"
#include "carpics.h"
#include "LogTrace.h"

using namespace log_functions;

//#include "finish_image.h"

#include <stdlib.h>
#include <stdio.h>

const int TURN_DELAY   = 1;
const int ACCEL_DELAY  = 5;
const int SMOOTH_VALUE = 0;

const char ORANGE[]     = "#FFC800";
const char YELLOW[]     = "#FFFF00";
const char BLUE[]       = "#00FFFF";
const char RED[]        = "#FF0000";
const char DARK_BLUE[]  = "#0000FF";
const char PURPLE[]     = "#C800FF";


/*****************************************************************************/
/* Function:     ZScreen                                                     */
/* Description:  Constructor                                                 */
/* Limitations:  None                                                        */
/* Return Value: None                                                        */
/*****************************************************************************/
ZScreen::ZScreen(QCanvas& c, QWidget* parent, const char* name)
   : QWidget( parent, name ),
     canvas(c)
{
   turnTime         = 0;
   AccelTime        = 0;
   TrackSize        = 0;
   CompAccelDelay   = 0;
   NumberOfLaps     = 0;
   FrameIndex       = 0;
   RaceCompleteFlag = false;
   RaceStarted      = false;

   TrackList.setAutoDelete(true);
   RaceCarList.setAutoDelete(true);

   BlueCar = new RaceCarImage((char *)BLUE);
   YellowCar = new RaceCarImage((char *)YELLOW);
   
   editor = new QCanvasView(&canvas, this);
   editor->setGeometry(8, -3, 225, 292);
   editor->setVScrollBarMode(QScrollView::AlwaysOff);
   editor->setHScrollBarMode(QScrollView::AlwaysOff);
   editor->setFrameStyle( QFrame::NoFrame );

   LapsHeader   = new QCanvasText(&canvas);
   UserLaps     = new QCanvasText(&canvas);
   ComputerLaps = new QCanvasText(&canvas);
   StartCounter = new QCanvasText(&canvas);

   UserLaps->setColor(QColor(0x00, 0xFF, 0xFF));
   UserLaps->setFont(QFont("Times", 12, QFont::Bold));

   LapsHeader->setColor(QColor(0xFF, 0xFF, 0xFF));
   LapsHeader->setFont(QFont("Times", 12, QFont::Normal));

   ComputerLaps->setColor(QColor(0xFF, 0xFF, 0x00));
   ComputerLaps->setFont(QFont("Times", 12, QFont::Bold));

   StartCounter->setColor(QColor(0x00, 0xFF, 0x00));
   StartCounter->setFont(QFont("Times", 24, QFont::Bold));

   LapsHeader->setX(80);  // Gets Reset at Initialization
   LapsHeader->setY(5);
   LapsHeader->setZ(2);
   LapsHeader->hide();

   UserLaps->setX(10);
   UserLaps->setY(5);
   UserLaps->setZ(2);
   UserLaps->hide();

   ComputerLaps->setX(canvas.width()-20);
   ComputerLaps->setY(5);
   ComputerLaps->setZ(2);
   ComputerLaps->hide();

   StartCounter->setX(canvas.width()/2);
   StartCounter->setY(canvas.height()/2);
   StartCounter->setZ(4);
   StartCounter->hide();

   UserCarLine = new CarLine(&canvas);

   AcceptKeyPressEvent = true;
}

/*****************************************************************************/
/* Function:     Initialize                                                  */ 
/* Description:  Initializes the variables at the start of a new race        */
/* Limitations:  None                                                        */
/* Return Value: None                                                        */
/*****************************************************************************/
void ZScreen::Initialize(int Track, int Level, int Laps)
{
   START_FUNCTION_TRACE("ZScreen::Initialize");
   QRect textRect;
//   QTime cTime = QTime::currentTime();

   turnTime            = 0;
   AccelTime           = 0;
   RaceState           = 0;
   NumberOfLaps        = Laps;
   RaceCompleteFlag    = false;
   ElapsedTime         = 0;
   ElapsedSeconds      = 0;
   ElapsedMinutes      = 0;
   TotalTime           = 0;

   TurnLeftEnabled     = false;
   TurnRightEnabled    = false;
   AcceptKeyPressEvent = false;

   LoadTrack(Track);

   addSprite("BlueCar", USER_CONTROL, Level);
   addSprite("YellowCar", COMPUTER_CONTROL, Level);

   LapsHeader->setText("Laps Completed");
   textRect = LapsHeader->boundingRect();
   LapsHeader->setX( (canvas.width()/2) - (textRect.width()/2) );
   LapsHeader->show();

   UserLaps->setText("0");
   UserLaps->show();

   ComputerLaps->setText("0");
   ComputerLaps->show();

   StartCounter->setText("3");
   textRect = StartCounter->boundingRect();
   StartCounter->setX( (canvas.width()/2) - (textRect.width()/2) );
   StartCounter->setY( (canvas.height()/2) - (textRect.height()/2) );
   StartCounter->show();

   timer = new QTimer( this );
   connect( timer, SIGNAL(timeout()), this, SLOT(TimeEvent()) );
   timer->start( 1000, false );
   END_FUNCTION_TRACE("ZScreen::Initialize");
}

/*****************************************************************************/
/* Function:     TimeEvent                                                   */
/* Description:  Controls the transition of the various stages of a game.    */
/* Limitations:  None                                                        */
/* Return Value: None                                                        */
/*****************************************************************************/
void ZScreen::TimeEvent()
{
   START_FUNCTION_TRACE("ZScreen::TimeEvent");
   QRect textRect;
   static int holdGo = 0;

   switch (RaceState)
   {
      case 0:
         RaceState = 1;
         StartCounter->setText("2");
         textRect = StartCounter->boundingRect();
         StartCounter->setX( (canvas.width()/2) - (textRect.width()/2) );
         StartCounter->setY( (canvas.height()/2) - (textRect.height()/2) );
         StartCounter->show();
         canvas.update();
         break;

      case 1:
         RaceState = 2;
         StartCounter->hide();
         StartCounter->setText("1");
         StartCounter->show();
         canvas.update();
         break;

      case 2:
         RaceState = 3;
         StartCounter->hide();
         StartCounter->setText("GO!");
         textRect = StartCounter->boundingRect();
         StartCounter->setX( (canvas.width()/2) - (textRect.width()/2) );
         StartCounter->setY( (canvas.height()/2) - (textRect.height()/2) );
         StartCounter->show();
         holdGo = 0;
         RaceStarted = false;
         timer->changeInterval(REFRESH_RATE);
         canvas.update();
         break;

      case 3:
         if (holdGo > 10)
         {
            StartCounter->hide();
         }
         else
         {
            holdGo++;
         }

         if (RaceCompleteFlag == false)
         {
            RunTheRace();
            UpdateElapsedRaceTime();
            canvas.advance();
         }
         else
         {
            timer->changeInterval(1500);
            RaceState = 4;
         }
         break;

      case 4:
         LapsHeader->hide();
         UserLaps->hide();
         UserCarLine->hide();
         ComputerLaps->hide();

         delete timer;
         RaceCarList.clear();
         TrackList.clear();
         emit RaceOver(theWinner, TotalTime);
         break;
   }
   END_FUNCTION_TRACE("ZScreen::TimeEvent");
}

/*****************************************************************************/
/* Function:     UpdateElapsedRaceTime                                       */
/* Description:                                                              */
/* Limitations:  None                                                        */
/* Return Value: None                                                        */
/*****************************************************************************/
void ZScreen::UpdateElapsedRaceTime()
{
   START_FUNCTION_TRACE("ZScreen::UpdateElapsedRaceTime");
   char Milliseconds[4] = {'\0'};
   char Seconds[3] = {'\0'};
   char Minutes[3] = {'\0'};

   TotalTime   = TotalTime + REFRESH_RATE;

   ElapsedTime = ElapsedTime + REFRESH_RATE;
   if (ElapsedTime >= 1000)
   {
      ElapsedTime = ElapsedTime - 1000;
      ElapsedSeconds = ElapsedSeconds + 1;
      if (ElapsedSeconds >= 60)
      {
         ElapsedSeconds = ElapsedSeconds - 60;
         ElapsedMinutes = ElapsedMinutes + 1;
      }
   }

   // Format the Elapsed Time String using leading zeros if
   // number less than 1000.
   if (ElapsedTime < 10)
   {
      sprintf(Milliseconds, "%s%d", "00", ElapsedTime);
   }
   else if (ElapsedTime < 100)
   {
      sprintf(Milliseconds, "%c%d", '0', ElapsedTime);
   }
   else
   {
      sprintf(Milliseconds, "%d", ElapsedTime);
   }

   // Format the Elapsed Time String using leading zeros if
   // number less than 1000.
   if (ElapsedSeconds < 10)
   {
      sprintf(Seconds, "%c%d", '0', ElapsedSeconds);
   }
   else
   {
      sprintf(Seconds, "%d", ElapsedSeconds);
   }

   // Format the Elapsed Time String using leading zeros if
   // number less than 10.
   if (ElapsedMinutes < 10)
   {
      sprintf(Minutes, "%c%d", '0', ElapsedMinutes);
   }
   else
   {
      sprintf(Minutes, "%d", ElapsedMinutes);
   }
   ElapsedTimeString.sprintf("%s:%s:%s", Minutes, Seconds, Milliseconds);

   LapsHeader->setText(ElapsedTimeString);
   QRect textRect = LapsHeader->boundingRect();
   LapsHeader->setX( (canvas.width()/2) - (textRect.width()/2) );
   LapsHeader->show();
   END_FUNCTION_TRACE("ZScreen::UpdateElapsedRaceTime");
}

/*****************************************************************************/
/* Function:     RaceHasBeenWon                                              */
/* Description:                                                              */
/* Limitations:  None                                                        */
/* Return Value: None                                                        */
/*****************************************************************************/
void ZScreen::RaceHasBeenWon(CarControlType Winner)
{
   START_FUNCTION_TRACE("ZScreen::RaceHasBeenWon");
   if (Winner == COMPUTER_CONTROL)
   {
      theWinner = false;
   }
   else
   {
      theWinner = true;
   }
   
   RaceCar *raceCar;
   for (raceCar = RaceCarList.first(); raceCar; raceCar = RaceCarList.next())
   {
      raceCar->setVelocity(0,0);
      raceCar->SetRaceCarSpeed(0);
      raceCar->SetRaceCarDirection(0);
      raceCar->ResetRace();
   }
   turnTime            = 0;
   TrackSize           = 0;
   NumberOfLaps        = 0;

   RaceCompleteFlag    = true;
   AccelerateEnabled   = false;
   DecelerateEnabled   = false;
   TurnLeftEnabled     = false;
   TurnRightEnabled    = false;
   AcceptKeyPressEvent = false;
   END_FUNCTION_TRACE("ZScreen::RaceHasBeenWon");
}

/*****************************************************************************/
/* Function:     addSprite                                                   */
/* Description:  Adds a new car image to the Canvas Image.                   */
/* Limitations:  None                                                        */
/* Return Value: None                                                        */
/*****************************************************************************/
void ZScreen::addSprite(QString CarImage, CarControlType Control, int Difficulty)
{
   START_FUNCTION_TRACE("ZScreen::addSprite");
   QCanvasPixmapArray *aCar;

   if (CarImage == "BlueCar")
   {
      aCar = new QCanvasPixmapArray(BlueCar->RaceCarList, 
                                    BlueCar->UserHotSpots);
   }
   else
   {
      aCar = new QCanvasPixmapArray(YellowCar->RaceCarList, 
                                    YellowCar->ComputerHotSpots);
   }
      
   bool found = false;

   // Find the Start Track Piece.
   GridSection *trackPiece = TrackList.first();
   while ( (trackPiece != NULL ) && (found == false) )
   {
      if (trackPiece->IsStartTrack() == true)
      {
         found = true;
      }
      else
      {
         trackPiece = TrackList.next();
      }
   }

   if (found)
   {
      // Add the current car to the Race Car List.  After Initialization
      // is complete Race Car List should contain two cars, a User Controlled
      // and a Computer Controlled car.  This may change in the future and a
      // third car may be added.
      RaceCarList.append(new RaceCar(aCar, &canvas, Control));

      RaceCarList.current()->setAnimated(true);
      if (Control == COMPUTER_CONTROL)
      {
         if (Difficulty != 1)
         {
            RaceCarList.current()->SetRaceCarMaxSpeed(4);
         }
         else
         {
            RaceCarList.current()->SetRaceCarMaxSpeed(5);
         }

         // Offset needs to be added (-10, +10) to accomodate for the fact
         // that the Computer Controlled cars Hot Spot is located in the
         // center of the car.  The User Controlled cars Hot Spot is located
         // at the front of the car.
         if (trackPiece->TrackImageID == HORIZ_START)
         {
            RaceCarList.current()->setX(trackPiece->GetGridMaxXValue());
            RaceCarList.current()->setY(trackPiece->GetGridMinYValue() + 30);
            RaceCarList.current()->SetRaceCarDirection(6);
            RaceCarList.current()->setFrame(18);
         }
         else
         {
            RaceCarList.current()->setX(trackPiece->GetGridMinXValue() + 30);
            RaceCarList.current()->setY(trackPiece->GetGridMaxYValue());
            RaceCarList.current()->SetRaceCarDirection(0);
            RaceCarList.current()->setFrame(0);
         }
      }
      else
      {
         if ( (Difficulty == 0) || (Difficulty == 1) )
         {
            RaceCarList.current()->IgnoreCollisions(true);
         }
         else
         {
            RaceCarList.current()->IgnoreCollisions(false);
         }

         if (trackPiece->TrackImageID == HORIZ_START)
         {
            RaceCarList.current()->setX(trackPiece->GetGridMaxXValue() - 10);
            RaceCarList.current()->setY(trackPiece->GetGridMinYValue() + 40);
            RaceCarList.current()->SetRaceCarDirection(18);
            RaceCarList.current()->setFrame(18);
         }
         else
         {
            RaceCarList.current()->setX(trackPiece->GetGridMinXValue() + 40);
            RaceCarList.current()->setY(trackPiece->GetGridMaxYValue() - 10);
            RaceCarList.current()->SetRaceCarDirection(0);
            RaceCarList.current()->setFrame(0);
         }
      }

      // Set the Z Access level of the cars to three.  This is in case
      // the Start Track is a bridge piece.  We will not know this until
      // the race actually starts so we just assume it to be for now.
      RaceCarList.current()->setZ(3);

      if (Control == COMPUTER_CONTROL)
      {
         // Don't have a way of recalculating Curve Points when we change
         // the speed while turning.
         if (trackPiece->TrackImageID == RIGHT_UPPER_BRIDGE_START)
         {
            RaceCarList.current()->SetRaceCarSpeed(
                                    RaceCarList.current()->GetRaceCarMaxSpeed());
         }
         else
         {
            RaceCarList.current()->SetRaceCarSpeed(0);
         }
      }
      else
      {
         RaceCarList.current()->SetRaceCarSpeed(0);
      }

      RaceCarList.current()->show();
   }
   END_FUNCTION_TRACE("ZScreen::addSprite");
}

/*****************************************************************************/
/* Function:     RestartRaceCar                                              */
/* Description:  Restarts the Car Sprite after the car runs off the screen.  */
/* Limitations:  None                                                        */
/* Return Value: None                                                        */
/*****************************************************************************/
void ZScreen::RestartRaceCar(RaceCar *raceCar)
{
   START_FUNCTION_TRACE("ZScreen::RestartRaceCar");
   bool found = false;

   GridSection *trackPiece = TrackList.first();
   while ( (trackPiece != NULL ) && (found == false) )
   {
      if (trackPiece->IsStartTrack() == true)
      {
         found = true;
      }
      else
      {
         trackPiece = TrackList.next();
      }
   }
   
   if (found)
   {
      if (raceCar->GetCarControl() == COMPUTER_CONTROL)
      {
         raceCar->setX(trackPiece->GetGridMinXValue() + 30);
         raceCar->setY(trackPiece->GetGridMaxYValue());
         CurrentComputerTrack = trackPiece;
      }
      else
      {
         raceCar->setX(trackPiece->GetGridMinXValue() + 40);
         raceCar->setY(trackPiece->GetGridMaxYValue() - 10);
         UserCarLine->hide();
         CurrentUserTrack  = trackPiece;
      }
      raceCar->RestartFrameCounter = 0;
      raceCar->SetRaceCarSpeed(0);
      raceCar->setZ(3);
      raceCar->SetRaceCarDirection(0);
      raceCar->setFrame(0);
      raceCar->setVelocity( 0, 0 );
      raceCar->show();
      raceCar->SetRestartFlag(true);
      
      AccelerateEnabled   = false;
      DecelerateEnabled   = false;
      TurnLeftEnabled     = false;
      TurnRightEnabled    = false;
      AcceptKeyPressEvent = false;
   }
   END_FUNCTION_TRACE("ZScreen::RestartRaceCar");
}

/*****************************************************************************/
/* Function:     keyReleaseEvent                                             */ 
/* Description:  Receives Key Release events which are used to update the    */
/*               cars direction and speed.                                   */
/* Limitations:  None                                                        */
/* Return Value: None                                                        */
/*****************************************************************************/
void ZScreen::keyReleaseEvent( int key )
{
   START_FUNCTION_TRACE("ZScreen::keyReleaseEvent");
   if (AcceptKeyPressEvent)
   {
      AcceptKeyPressEvent = false;

      switch ( key ) 
      {
         case Key_Up:
            AccelerateEnabled = false;
            break;
            
         case Key_Down:
            DecelerateEnabled = false;
            break;
            
         case Key_Left:
            TurnLeftEnabled = false;
            break;
   
         case Key_Right:
            TurnRightEnabled = false;
            break;
      }

      AcceptKeyPressEvent = true;
   }
   END_FUNCTION_TRACE("ZScreen::keyReleaseEvent");
}

/*****************************************************************************/
/* Function:     keyPressEvent                                               */ 
/* Description:  Receives Key Press events which are used to update the      */
/*               cars direction and speed.                                   */
/* Limitations:  None                                                        */
/* Return Value: None                                                        */
/*****************************************************************************/
void ZScreen::keyPressEvent( int key )
{
   START_FUNCTION_TRACE("ZScreen::keyPressEvent");
   RaceCar *raceCar;
   
   if (AcceptKeyPressEvent)
   {
      AcceptKeyPressEvent = false;

      switch ( key ) 
      {
         case Key_Up:
            AccelerateEnabled = true;
            break;

         case Key_Down:
            DecelerateEnabled = true;
            break;
   
         case Key_Left:
            TurnLeftEnabled = true;
            break;
   
         case Key_Right:
            TurnRightEnabled = true;
            break;
   
         case Key_R:
            raceCar = RaceCarList.first();
            while ( (raceCar) && (raceCar->GetCarControl() != COMPUTER_CONTROL) )
            {
               raceCar = RaceCarList.next();
               
               if (raceCar->GetCarControl() == COMPUTER_CONTROL)
               {
                  RestartRaceCar(raceCar);
               }
            }
            break;
         
         case Key_Q:                               // quit
            QApplication::exit();
            break;
/*
         case Key_Return:
         case Key_Enter:
            if (RaceCompleteFlag)
            {
               RaceCarList.clear();
               TrackList.clear();
               delete FinishImage;
               emit RaceOver();
            }
            break;
*/
         case Key_Escape:
            RaceHasBeenWon(COMPUTER_CONTROL);
            break;
            
         default:
/*
            if (RaceCompleteFlag)
            {
               RaceCarList.clear();
               TrackList.clear();
               delete FinishImage;
               emit RaceOver();
            }
*/
            break;
      }
      
      AcceptKeyPressEvent = true;
   }
   END_FUNCTION_TRACE("ZScreen::keyPressEvent");
}

/*****************************************************************************/
/* Function:     UpdateUserCarSpeed                                          */ 
/* Description:  Changes the speed of the User Controlled car based on       */ 
/*               current key press events.                                   */
/* Limitations:  None                                                        */
/* Return Value: None                                                        */
/*****************************************************************************/
void ZScreen::UpdateUserCarSpeed(RaceCar *UserCar)
{
   START_FUNCTION_TRACE("ZScreen::UpdateUserCarSpeed");
   if ( (AccelerateEnabled == true) || (DecelerateEnabled == true) )
   {
      if (AccelTime)
      {
         AccelTime--;
      }
      else
      {
         if (AccelerateEnabled)
         {
            if (UserCar->GetRaceCarSpeed() < MAX_SPEED)
            {
               UserCar->SetRaceCarSpeed(UserCar->GetRaceCarSpeed() + 1);
            }
         }
         else
         {
            if (UserCar->GetRaceCarSpeed() > 0)
            {
               UserCar->SetRaceCarSpeed(UserCar->GetRaceCarSpeed() - 1);
            }
         }

         // The delay is used to keep the car from accelerating too fast.
         // This helps create a more realistic feel to the handling.
         AccelTime = ACCEL_DELAY;
      }
   }
   END_FUNCTION_TRACE("ZScreen::UpdateUserCarSpeed");
}

   
/*****************************************************************************/
/* Function:     UpdateUserCarDirection                                      */ 
/* Description:  Changes the direction of the User Controlled car based on   */ 
/*               current key press events.                                   */
/* Limitations:  None                                                        */
/* Return Value: None                                                        */
/*****************************************************************************/
void ZScreen::UpdateUserCarDirection(RaceCar *UserCar)
{
   START_FUNCTION_TRACE("ZScreen::UpdateUserCarDirection");
   if ( (TurnLeftEnabled == true) || (TurnRightEnabled == true) )
   {
      if (turnTime)
      {
         turnTime--;
      }
      else
      {
         if (TurnLeftEnabled)
         {
            if (UserCar->GetRaceCarDirection() == 0)
            {
               UserCar->SetRaceCarDirection(23);
            }
            else
            {
               UserCar->SetRaceCarDirection(UserCar->GetRaceCarDirection() - 1);
            }
         }
         else
         {
            if (UserCar->GetRaceCarDirection() == 23)
            {
               UserCar->SetRaceCarDirection(0);
            }
            else
            {
               UserCar->SetRaceCarDirection(UserCar->GetRaceCarDirection() + 1);
            }
         }

         // A Delay is added to keep the car from turning too quickly.  The
         // delay value is set based on the speed of the Race Car.  This
         // helps create more realistic handling.
         if (UserCar->GetRaceCarSpeed() < 3)
         {
            turnTime = TURN_DELAY + 2;
         }
         else if (UserCar->GetRaceCarSpeed() == 3)
         {
            turnTime = TURN_DELAY + 1;
         }
         else
         {
            turnTime = TURN_DELAY;                  
         }
      }
   }
   END_FUNCTION_TRACE("ZScreen::UpdateUserCarDirection");
}

   
/*****************************************************************************/
/* Function:     GetRaceCarFront                                             */ 
/* Description:  Determines the front point of the User Controlled Race Car. */
/* Limitations:  None                                                        */
/* Return Value: None                                                        */
/*****************************************************************************/
void ZScreen::GetRaceCarFront(RaceCar *car, double *dX, double *dY)
{
   START_FUNCTION_TRACE("ZScreen::GetRaceCarFront");
   double CarWidth  = car->GetWidth();
   double CarHeight = car->GetHeight();

   double WidthOffset  = (CarWidth/6);
   double HeightOffset = (CarHeight/6);

   switch (car->GetRaceCarPicIndex())
   {
      case 0:
         *dX = *dX + (CarWidth/2);
         break;

      case 1:
         *dX = *dX + (CarWidth/2) + WidthOffset;
         break;

      case 2:
         *dX = *dX + (CarWidth/2) + (WidthOffset*2);
         break;

      case 3:
         *dX = *dX + CarWidth;
         break;

      case 4:
         *dX = *dX + CarWidth;
         *dY = *dY + HeightOffset;
         break;

      case 5:
         *dX = *dX + CarWidth;
         *dY = *dY + (HeightOffset*2);
         break;

      case 6:
         *dX = *dX + CarWidth;
         *dY = *dY + (CarHeight/2);
         break;

      case 7:
         *dX = *dX + CarWidth;
         *dY = *dY + (CarHeight/2) + HeightOffset;
         break;

      case 8:
         *dX = *dX + CarWidth;
         *dY = *dY + (CarHeight/2) + (HeightOffset*2);
         break;

      case 9:
         *dX = *dX + CarWidth;
         *dY = *dY + CarHeight;
         break;

      case 10:
         *dX = *dX + (CarWidth/2) + (WidthOffset*2);
         *dY = *dY + CarHeight;
         break;

      case 11:
         *dX = *dX + (CarWidth/2) + WidthOffset;
         *dY = *dY + CarHeight;
         break;

      case 12:
         *dX = *dX + (CarWidth/2);
         *dY = *dY + CarHeight;
         break;

      case 13:
         *dX = *dX + (WidthOffset*2);
         *dY = *dY + CarHeight;
         break;

      case 14:
         *dX = *dX + WidthOffset;
         *dY = *dY + CarHeight;
         break;

      case 15:
         *dY = *dY + CarHeight;
         break;

      case 16:
         *dY = *dY + (CarHeight/2) + (HeightOffset*2);
         break;

      case 17:
         *dY = *dY + (CarHeight/2) + HeightOffset;
         break;

      case 18:
         *dY = *dY + (CarHeight/2);
         break;

      case 19:
         *dY = *dY + (HeightOffset*2);;
         break;

      case 20:
         *dY = *dY + HeightOffset;
         break;

      case 21:
         break;

      case 22:
         *dX = *dX + WidthOffset;
         break;

      case 23:
         *dX = *dX + (WidthOffset*2);
         break;
   }
   END_FUNCTION_TRACE("ZScreen::GetRaceCarFront");
}


/*****************************************************************************/
/* Function:     GetCurrentGridSection                                       */ 
/* Description:  Searches for the current Grid Section that the Race Car is  */
/*               located on.                                                 */
/* Limitations:  None                                                        */
/* Return Value: None                                                        */
/*****************************************************************************/
GridSection *ZScreen::GetCurrentGridSection(double DX, double DY, double DZ)
{
   START_FUNCTION_TRACE("ZScreen::GetCurrentGridSection");
   bool found = false;

   GridSection *trackPiece = TrackList.first();

   // Search for the track piece on the current layer
   while ( (trackPiece != NULL ) && (found == false) )
   {
      // Search the current Layer first
      if (trackPiece->OnGrid((int)DX, (int)DY, (int)DZ) != 0)
      {
         trackPiece = TrackList.next();
      }
      else
      {
         found = true;
      }
   }
   
   // If the Current Track piece is not found on the Current Layer
   // search the next layer up or down.
   if (found == false)
   {
      trackPiece = TrackList.first();
      while ( (trackPiece != NULL ) && (found == false) )
      {
         // If not on the upper Layer then Search the Lower Layer
         if (DZ == 3)
         {
            if (trackPiece->OnGrid((int)DX, (int)DY, (int)(DZ-2)) != 0)
            {
               trackPiece = TrackList.next();
            }
            else
            {
               found = true;
            }
         }
         else
         {
            // If not on the Lower Layer then Search the Upper Layer
            if (trackPiece->OnGrid((int)DX, (int)DY, (int)(DZ+2)) != 0)
            {
               trackPiece = TrackList.next();
            }
            else
            {
               found = true;
            }
         }
      }
   }

   END_FUNCTION_TRACE("ZScreen::GetCurrentGridSection");
   return trackPiece;
}


/*****************************************************************************/
/* Function:     RunTheRace                                                  */
/* Description:  Controls the movement of the cars around the track          */
/* Limitations:  None                                                        */
/* Return Value: None                                                        */
/*****************************************************************************/
void ZScreen::RunTheRace()
{
   START_FUNCTION_TRACE("ZScreen::RunTheRace");
   QString textString;
   double dX, dY, dZ;
   QRect  centerline;
   
   int    TrackTotal  = 0;

   int    localX      = 0;
   int    localY      = 0;
   bool   underBridge = false;

   RaceCar *raceCar;
   for (raceCar = RaceCarList.first(); raceCar; raceCar = RaceCarList.next())
   {
      dX = raceCar->x();
      dY = raceCar->y();
      dZ = raceCar->z();
      
      if ( raceCar->GetLapsCompleted() >= NumberOfLaps )
      {
         raceCar->setVelocity(0,0);
         RaceHasBeenWon(raceCar->GetCarControl());
         continue;
      }

      if (raceCar->GetCarControl() == USER_CONTROL)
      {
         GetRaceCarFront(raceCar, &dX, &dY);
      }

      GridSection *trackPiece = GetCurrentGridSection(dX, dY, dZ);

      if (trackPiece)
      {
         trackPiece->GetGridXYValues(dX, dY, &localX, &localY);
         
         if (raceCar->GetCarControl() != USER_CONTROL)
         {
            if (raceCar->GetRestartFlag() == false)
            {
               if (raceCar->GetRaceCarSpeed() > 0)
               {
                  if (RaceStarted == true)
                  {
                     raceCar->UpdateComputerControlCar(localX, localY, trackPiece);
                  }
                  else
                  {
                     switch ( raceCar->GetRaceCarDirection() ) 
                     {
                        case 0:
                           raceCar->setVelocity( 0, (-1*raceCar->GetRaceCarSpeed()) );
                           break;

                        case 6:
                           raceCar->setVelocity( (-1*raceCar->GetRaceCarSpeed()), 0 );
                           break;

                        default:
                           break;
                     }
                  }
               }

               if ( raceCar->GetRaceCarSpeed() < raceCar->GetRaceCarMaxSpeed() )
               {
                  if (CompAccelDelay++ >= ACCEL_DELAY)
                  {
                     raceCar->SetRaceCarSpeed(raceCar->GetRaceCarSpeed() + 1);
                     CompAccelDelay = 0;
                  }
               }
            }

            if (trackPiece != CurrentComputerTrack)
            {
               raceCar->ClearTurnDelay();

               if (trackPiece->GetTrackValue() < 16)
               {
                  raceCar->TrackParts[trackPiece->GetTrackValue()] = 1;
               }

               if (trackPiece->IsStartTrack() == true)
               {
                  if (RaceStarted == false)
                  {
                     RaceStarted = true;
                  }
                  
                  for (int i = 0; i < 16; i++)
                  {
                     TrackTotal = TrackTotal + raceCar->TrackParts[i];
                  }
                  if (TrackTotal == TrackSize)
                  {
                     raceCar->UpdateLapsCompleted();
                  }
               }
               CurrentComputerTrack = trackPiece;

               textString = textString.setNum(raceCar->GetLapsCompleted());
               ComputerLaps->setText(textString);
            }
         }
         else
         {
            // Update the current User Car controls based on recent key press events.
            // Check to see if car is under bridge piece.
            UpdateUserCarSpeed(raceCar);
            UpdateUserCarDirection(raceCar);

            // Check to see if the User Car is currently being drawn under another
            // track piece (going under a bridge). 
            if (dZ == 1)
            {
               underBridge = false;

               GridSection *bridgePiece = TrackList.first();
               while ( (bridgePiece != NULL ) && (underBridge == false) )
               {
                  // If not on the Lower Layer then Search the Upper Layer
                  if (bridgePiece->OnGrid((int)dX, (int)dY, (int)(dZ+2)) != 0)
                  {
                     bridgePiece = TrackList.next();
                  }
                  else
                  {
                     underBridge = true;
                  }
               }
            }

            raceCar->UpdateUserControlCar(localX, localY, trackPiece);

            // If the car is currently being drawn under another track piece
            // display a line over the car and over the track piece so that
            // it is possible to tell where the car is going.
            if (underBridge)
            {
               centerline = raceCar->GetRaceCarCenterline();
               UserCarLine->SetPoints(centerline);
               UserCarLine->show();
            }
            else
            {
               // We probably should find another way of doing this instead
               // of calling this function every time we update the car.  In
               // most instances the User Car Line will already be hidden.
               UserCarLine->hide();
            }

            // The number of laps are tracked by checking to see if the
            // car has passed through all of the current track pieces.
            // The number of track pieces traveled through is checked
            // every time we transition to a new track piece.
            if (trackPiece != CurrentUserTrack)
            {
               raceCar->TrackParts[trackPiece->GetTrackValue()] = 1;
               
               if (trackPiece->IsStartTrack() == true)
               {
                  // I shouldn't be using Hard Coded numbers.  This (16) needs
                  // to be changed --kog.
                  for (int i = 0; i < 16; i++)
                  {
                     TrackTotal = TrackTotal + raceCar->TrackParts[i];
                     raceCar->TrackParts[i] = 0;
                  }

                  // If all track pieces have been crossed and we are currently
                  // on a start track piece (back where we started the race).
                  if (TrackTotal == TrackSize)
                  {
                     // Update the Lap Counter
                     raceCar->UpdateLapsCompleted();
                  }
               }
               CurrentUserTrack = trackPiece;

               // It shouldn't be necessary to do this every time we go to a
               // new grid location.  It should only be done when we update the
               // lap counter --kog.
               textString = textString.setNum(raceCar->GetLapsCompleted());
               UserLaps->setText(textString);
            }

            if ( (raceCar->GetRestartFlag() == false) && 
                 (AcceptKeyPressEvent == false) )
            {
               AcceptKeyPressEvent = true;
            }
         }

         if (raceCar->GetRestartFlag() == false)
         {
            if (raceCar->CheckForCollision() > MAX_COLLISIONS)
            {
               if ( (FrameIndex >= 0) && (FrameIndex < 23) )
               {
                  raceCar->setFrame(FrameIndex);
                  FrameIndex++;
               }
               else
               {
                  raceCar->ClearCollisionFlag();
                  FrameIndex = 0;
               }
            }
            else
            {
               if ( (raceCar->GetRaceCarPicIndex() >= 0) && 
                    (raceCar->GetRaceCarPicIndex() < 24) )
               {
                  raceCar->setFrame(raceCar->GetRaceCarPicIndex());
               }
               else
               {
                  raceCar->setFrame(0);
               }
            }
         }
         else
         {
            if ( (raceCar->RestartFrameCounter ) < 10)
            {
               raceCar->show();
            }
            else if ( (raceCar->RestartFrameCounter ) < 20)
            {
               raceCar->hide();
            }
            else if ( (raceCar->RestartFrameCounter ) < 30)
            {
               raceCar->show();
            }
            else
            {
               raceCar->SetRestartFlag(false);
            }
            raceCar->RestartFrameCounter++;
         }
      }
      else
      {
         RestartRaceCar(raceCar);
      }
   }
   END_FUNCTION_TRACE("ZScreen::RunTheRace");
}

/*****************************************************************************/
/* Function:     LoadTrack                                                    */ 
/* Description:  Adds the Track Image to the Canvas.                         */
/* Limitations:  None                                                        */
/* Return Value: None                                                        */
/*****************************************************************************/
void ZScreen::LoadTrack(int TrackImage)
{
   START_FUNCTION_TRACE("ZScreen::TrackImage");
   int i, j, k;

   TrackComponents Track;

   switch (TrackImage)
   {
      case 0:
         memcpy(Track.TrackImage, TrackImage1, sizeof(TrackImage1));
         memcpy(Track.TrackDirection, TrackDirection1, sizeof(TrackDirection1));
         TrackSize = TrackSize1;
         break;

      case 1:
         memcpy(Track.TrackImage, TrackImage2, sizeof(TrackImage2));
         memcpy(Track.TrackDirection, TrackDirection2, sizeof(TrackDirection2));
         TrackSize = TrackSize2;
         break;
   
      case 2:
         memcpy(Track.TrackImage, TrackImage3, sizeof(TrackImage3));
         memcpy(Track.TrackDirection, TrackDirection3, sizeof(TrackDirection3));
         TrackSize = TrackSize3;
         break;

      case 3:
         memcpy(Track.TrackImage, TrackImage4, sizeof(TrackImage4));
         memcpy(Track.TrackDirection, TrackDirection4, sizeof(TrackDirection4));
         TrackSize = TrackSize4;
         break;

      case 4:
         memcpy(Track.TrackImage, TrackImage5, sizeof(TrackImage5));
         memcpy(Track.TrackDirection, TrackDirection5, sizeof(TrackDirection5));
         TrackSize = TrackSize5;
         break;

      case 5:
         memcpy(Track.TrackImage, TrackImage6, sizeof(TrackImage6));
         memcpy(Track.TrackDirection, TrackDirection6, sizeof(TrackDirection6));
         TrackSize = TrackSize6;
         break;

      case 6:
         memcpy(Track.TrackImage, TrackImage7, sizeof(TrackImage7));
         memcpy(Track.TrackDirection, TrackDirection7, sizeof(TrackDirection7));
         TrackSize = TrackSize7;
         break;

      case 7:
         memcpy(Track.TrackImage, TrackImage8, sizeof(TrackImage8));
         memcpy(Track.TrackDirection, TrackDirection8, sizeof(TrackDirection8));
         TrackSize = TrackSize8;
         break;

      case 8:
         memcpy(Track.TrackImage, TrackImage9, sizeof(TrackImage9));
         memcpy(Track.TrackDirection, TrackDirection9, sizeof(TrackDirection9));
         TrackSize = TrackSize9;
         break;

      case 9:
         memcpy(Track.TrackImage, TrackImage10, sizeof(TrackImage10));
         memcpy(Track.TrackDirection, TrackDirection10, sizeof(TrackDirection10));
         TrackSize = TrackSize10;
         break;

      case 10:
         memcpy(Track.TrackImage, TrackImage11, sizeof(TrackImage11));
         memcpy(Track.TrackDirection, TrackDirection11, sizeof(TrackDirection11));
         TrackSize = TrackSize11;
         break;

      case 11:
         memcpy(Track.TrackImage, TrackImage12, sizeof(TrackImage12));
         memcpy(Track.TrackDirection, TrackDirection12, sizeof(TrackDirection12));
         TrackSize = TrackSize12;
         break;

      case 12:
         memcpy(Track.TrackImage, TrackImage13, sizeof(TrackImage13));
         memcpy(Track.TrackDirection, TrackDirection13, sizeof(TrackDirection13));
         TrackSize = TrackSize13;
         break;

      case 13:
         memcpy(Track.TrackImage, TrackImage14, sizeof(TrackImage14));
         memcpy(Track.TrackDirection, TrackDirection14, sizeof(TrackDirection14));
         TrackSize = TrackSize14;
         break;

      case 14:
         memcpy(Track.TrackImage, TrackImage15, sizeof(TrackImage15));
         memcpy(Track.TrackDirection, TrackDirection15, sizeof(TrackDirection15));
         TrackSize = TrackSize15;
         break;
   }

   int trackValue = 0;
   
   for (i = 0; i < ROWS; i++)
   {
      for ( j = 0; j < COLUMNS; j++)
      {
         for (k = 0; k < LAYERS; k++)
         {
            if (Track.TrackImage[k][i][j] != UNDEFINED)
            {
               GridSection *trackPiece = new GridSection(&canvas);
               trackPiece->LoadImage(Track.TrackImage[k][i][j], 
                                     Track.TrackDirection[k][i][j], j, i, k);
               trackPiece->SetImage();
               if (Track.TrackImage[k][i][j] != GRASS)
               {
                  trackPiece->SetTrackValue(trackValue);
                  trackValue++;
               }
               else
               {
                  trackPiece->SetTrackValue(0xFF);
               }
               TrackList.append(trackPiece);
            }
         }
      }
   }
   END_FUNCTION_TRACE("ZScreen::TrackImage");
}

