/***************************************************************************
                          zscreen.h  -  description
                             -------------------
    begin                : Wed Aug 21 01:49:10 EDT 2002
    copyright            : (C) 2002 by Kevin Greenhaw
    email                : kevin_greenhaw@email.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef __ZSCREEN_H__
#define __ZSCREEN_H__

#include <qwidget.h>
#include <qpopupmenu.h>
#include <qintdict.h>
#include <qcanvas.h>
#include <qimage.h>
#include <qglobal.h>

#include "grid.h"
#include "racecar.h"

#if (QT_VERSION == 301)
   #include <qptrlist.h>
#else
   #include <qlist.h>
#endif

const int REFRESH_RATE = 30;

class RaceCarImage;

class ZScreen : public QWidget
{
    Q_OBJECT

public:
   ZScreen(QCanvas& c, QWidget *parent = 0, const char *name = 0);
   void Initialize(int Track, int Level, int Laps);
   void keyPressEvent( int key );
   void keyReleaseEvent( int key );
 
signals:
   void RaceOver(bool Winner, long RaceTime);

private slots:
   void TimeEvent();

private:
   QCanvas&       canvas;
   QCanvasView   *editor;

   QCanvasText   *UserLaps;
   QCanvasText   *ComputerLaps;
   QCanvasText   *LapsHeader;
   QCanvasText   *StartCounter;

#if (QT_VERSION == 301)
   QPtrList<RaceCar> RaceCarList;
   QPtrList<GridSection> TrackList;
#else
   QList<RaceCar> RaceCarList;
   QList<GridSection> TrackList;
#endif

   bool AcceptKeyPressEvent;
   bool RaceCompleteFlag;
   
   bool TurnLeftEnabled;
   bool TurnRightEnabled;
   bool AccelerateEnabled;
   bool DecelerateEnabled;
   bool RaceStarted;
   
   int turnTime;
   int AccelTime;
   int TrackSize;
   int CollisionFlag;
   int FrameIndex;
   int CompAccelDelay;

   short ElapsedTime;
   short ElapsedSeconds;
   short ElapsedMinutes;
   long  TotalTime;
   QString ElapsedTimeString;

   int NumberOfLaps;
   int RaceState;
   
   bool theWinner;
   
   RaceCarImage *BlueCar;
   RaceCarImage *YellowCar;
   CarLine      *UserCarLine;
   
   QTimer *timer;
   QRect OpponentCar;
   
   GridSection *CurrentComputerTrack;
   GridSection *CurrentUserTrack;

   QCanvasPolygonalItem *FinishImage;

   void RaceHasBeenWon(CarControlType Winner);
   void LoadTrack(int TrackImage);
   void addSprite(QString CarImage, CarControlType Control, int Difficulty);
   void RestartRaceCar(RaceCar *raceCar);
   void RunTheRace();
   void UpdateElapsedRaceTime();
   void GetRaceCarFront(RaceCar *car, double *XSpot, double *YSpot);
   void UpdateUserCarDirection(RaceCar *UserCar);
   void UpdateUserCarSpeed(RaceCar *UserCar);
   GridSection *GetCurrentGridSection(double DX, double DY, double DZ);


};

#endif
